package com.seeyon.apps.govdoc.manager.external;

import java.util.*;

import com.seeyon.apps.doc.api.DocApi;
import com.seeyon.apps.govdoc.constant.GovdocEnum;
import com.seeyon.apps.govdoc.manager.*;
import com.seeyon.apps.govdoc.mark.manager.GovdocMarkManager;
import com.seeyon.apps.govdoc.po.GovdocExchangeDetail;
import com.seeyon.apps.govdoc.po.GovdocExchangeMain;
import com.seeyon.apps.govdoc.util.GovdocUtil;
import com.seeyon.apps.govdoc.vo.GovdocDealVO;
import com.seeyon.apps.govdoc.vo.GovdocRepealVO;
import com.seeyon.apps.ocip.OCIPConstants;
import com.seeyon.apps.ocip.util.OrgUtil;
import com.seeyon.ctp.cap.api.manager.CAPFormManager;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.ModuleType;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.content.comment.Comment;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.permission.manager.PermissionManager;
import com.seeyon.ctp.common.permission.vo.PermissionVO;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.trace.enums.WorkflowTraceEnums;
import com.seeyon.ctp.common.workflowmanage.handler.WorkflowManageHandler;
import com.seeyon.ctp.common.workflowmanage.vo.WorkflowData;
import com.seeyon.ctp.form.bean.FormBean;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.*;
import com.seeyon.ctp.workflow.bo.WorkItemInfoBO;
import com.seeyon.ctp.workflow.wapi.WorkflowApiManager;
import com.seeyon.ocip.common.org.OrgMember;
import com.seeyon.ocip.common.organization.IOrganizationManager;
import com.seeyon.ocip.common.utils.Global;
import com.seeyon.ocip.online.OnlineChecker;
import com.seeyon.v3x.edoc.domain.EdocSummary;
import com.seeyon.v3x.edoc.listener.EdocWorkflowManageHandler;
import com.seeyon.v3x.edoc.manager.EdocStatManager;
import net.joinwork.bpm.definition.BPMHumenActivity;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class GovdocWorkflowManageHandler implements WorkflowManageHandler {

	private static final Log LOGGER = LogFactory.getLog(GovdocWorkflowManageHandler.class);
    
	public static final int SEND_CONDITION = 19; //发文
    public static final int REC_CONDITION = 20;  //收文
    public static final int SIGN_CONDITION = 21; //签报
    public static final int GOVSEND_CONDITION = 401; //新公文，发文
    public static final int GOVREC_CONDITION = 402;  //新公文，收文
    public static final int GOVSIGN_CONDITION = 404; //新公文，签报
    
    public static final int RUN = 0;
    public static final int STOP = 1;
    public static final int CANCEL = 2;
    public static final int FINISH = 3;
    
    public static final String COMMON_WORKFLOW = "self";   //自由流程
    public static final String TEMPLETE_WORKFLOW = "template"; //模板流程
	private DocApi docApi;
    private GovdocManager govdocManager;
    private GovdocListManager govdocListManager;
    private GovdocLockManager govdocLockManager;
    private GovdocSummaryManager govdocSummaryManager;
    private PermissionManager permissionManager;
    private AffairManager affairManager;
    private GovdocPubManager govdocPubManager;
	private EdocStatManager edocStatManager;
	private GovdocMarkManager govdocMarkManager;
    private GovdocExchangeManager govdocExchangeManager;
	private CAPFormManager capFormManager;
	private WorkflowApiManager wapi;
	private OrgManager orgManager;
	private IOrganizationManager organizationManager;
	private GovdocFormManager govdocFormManager;
	private boolean hasMutiPlugin = Boolean.valueOf(AppContext.getSystemProperty(OCIPConstants.HAS_MUTI_PLUGIN));//是否开启多G6功能
	@Override
    public ModuleType getModuleType() {
        return ModuleType.edoc;
    }
    
    /**
     * 流程管理查询（分页数据）
     * @throws com.seeyon.ctp.common.exceptions.BusinessException 
     */
    public FlipInfo selectPageWorkflowDataByCondition(Map<String,Object> conditionParam, FlipInfo fi) throws BusinessException{
        long accountId = AppContext.getCurrentUser().getAccountId();
        Map<String,Object> map = getParamMap(conditionParam);

        List<WorkflowData> list = govdocListManager.getAdminWfDataList(fi,map,accountId,true,fi);
        
        String con = (String)conditionParam.get("condition");
        int condition = ApplicationCategoryEnum.edocSend.key();
        if(Strings.isNotBlank(con)){
            condition = Integer.parseInt(con);
        }
        PermissionVO vo = null;
        if(Strings.isNotEmpty(list)){
        	for(WorkflowData data : list){
				//    edocSend(19), //发文
				//    edocRec(20),    //收文
				//    edocSign(21),    //签报
        		if("edocSend".equals(data.getNps()))
				{
					condition = 19;
				}
				if("edocRec".equals(data.getNps()))
				{
					condition = 20;
				}
				if("edocSign".equals(data.getNps()))
				{
					condition = 21;
				}
        		vo = permissionManager.getDefaultPermissionByApp(ApplicationCategoryEnum.valueOf(condition), data.getAccountId());
        		if(vo != null){
            		data.setDefaultNodeName(vo.getName());
            		data.setDefaultNodeLable(vo.getLabel());
        		}
        	}
        }
        fi.setData(list);
        return fi;
    }

    /**
     * 流程管理数据查询(全部数据，不分页)
     * @throws com.seeyon.ctp.common.exceptions.BusinessException 
     */
    public List<WorkflowData> selectWorkflowDataByCondition(Map<String,Object> conditionParam) throws BusinessException{
        long accountId = AppContext.getCurrentUser().getAccountId();
        Map<String,Object> map = getParamMap(conditionParam);

        Map<String,List<Long>> senderMap = (Map<String,List<Long>>)map.get("senderMap");
        List<WorkflowData> list = govdocListManager.getAdminWfDataList(null, map,accountId,false,null);
        return list;
    }

    @Override
    public String transRepal(Map<String, Object> map) throws BusinessException {
        String _affairId = (String) map.get("affairId");
        
        CtpAffair affair=affairManager.get(Long.valueOf(_affairId));
        long summaryId = affair.getObjectId();
        String repealComment = String.valueOf(map.get("repealComment"));
        
		if (affair.getSubApp()<=4) {// 新公文
			Map result = govdocManager.checkIsCanRepeal(map);
			if(result.containsKey("msg")){
				return result.get("msg").toString();
			}
			GovdocRepealVO vo = new GovdocRepealVO();
			vo.setSummaryIdStr(summaryId + "");
			vo.setAffairIdStr(_affairId);
			vo.setRepealComment(repealComment);
			govdocManager.transRepal(vo);
			return null;
		} else {
			EdocWorkflowManageHandler edocWorkflowManageHandler = (EdocWorkflowManageHandler)AppContext.getBean("edocWorkflowManageHandler");
			edocWorkflowManageHandler.transRepal(map);
		}
        return null;
    }

    @Override
    public String transStepStop(Map<String, Object> map) throws BusinessException {
    	long userId = AppContext.getCurrentUser().getId();
        String _affairId = (String) map.get("affairId");
        
        CtpAffair affair=affairManager.get(Long.valueOf(_affairId));
        long summaryId = affair.getObjectId();
        if (affair.getSubApp()<=4) {// 新公文
        	GovdocDealVO dealVo = new GovdocDealVO();
            dealVo.setCurrentDate(DateUtil.currentDate());
            dealVo.setCurrentUser(AppContext.getCurrentUser());
            Comment comment = new Comment();
            comment.setContent("流程管理终止");
    		dealVo.setAffairId(Long.valueOf(_affairId));
    		//dealVo.setExtAtt1(GovdocParamUtil.getString(request, "extAtt1"));
    		dealVo.setComment(comment);
    		dealVo.setFrom("batch");
    		govdocPubManager.fillStepStopObj(dealVo);
    			
    				
    		govdocManager.transStepStop(dealVo);

			
			return null;
		} else {
			EdocWorkflowManageHandler edocWorkflowManageHandler = (EdocWorkflowManageHandler)AppContext.getBean("edocWorkflowManageHandler");
			return edocWorkflowManageHandler.transStepStop(map);
		}
    }

    private Map getParamMap(Map<String,Object> conditionParam) {
        Date bDate = null;
        Date eDate = null;
        String beginDate = (String) conditionParam.get("beginDate");
        String endDate = (String) conditionParam.get("endDate");
        if(Strings.isNotBlank(beginDate)){
            bDate = Datetimes.getTodayFirstTime(beginDate);
        }
        if(Strings.isNotBlank(endDate)){
            eDate = Datetimes.getTodayLastTime(endDate);
        }
        String con = (String)conditionParam.get("condition");
        int condition = ApplicationCategoryEnum.collaboration.key();
        if(Strings.isNotBlank(con)){
            condition = Integer.parseInt(con);
        }
        int edocType = 0;
        if(condition == SEND_CONDITION || condition == GOVSEND_CONDITION){
            edocType = 0;
        }else if(condition == REC_CONDITION || condition == GOVREC_CONDITION){
            edocType = 1;
        }else if(condition == SIGN_CONDITION || condition == GOVSIGN_CONDITION){
            edocType = 2;
        }
        String sendersStr = (String) conditionParam.get("senders");
        String[] sendersArr = sendersStr.split(",");
        List<String> sendersList = new ArrayList<String>();
        if(sendersArr != null && sendersArr.length>0){
            for(int i=0; i<sendersArr.length; i++){
                if(!"".equals(sendersArr[i])){
                    sendersList.add(sendersArr[i]);
                }
            }
        }
        //将发起对象，按照各自的类别分别存放到List中，因为选人时，可以混选单位，部门，人员
        Map<String,List<Long>> senderMap = new HashMap<String,List<Long>>();
        for(int i=0;i<sendersList.size();i++){
            String sender = sendersList.get(i);
            String[] senders = sender.split("[|]");
            String type = senders[0];
            long id = Long.parseLong(senders[1]);
            if(senderMap.get(type)==null){
                List<Long> senderList = new ArrayList<Long>();
                senderList.add(id);
                senderMap.put(type, senderList);
            }else{
                List<Long> senderList = senderMap.get(type);
                senderList.add(id);
                senderMap.put(type, senderList);
            }
        }
        
        conditionParam.put("bDate", bDate);
        conditionParam.put("eDate", eDate);
        conditionParam.put("edocType", edocType);
        conditionParam.put("senderMap", senderMap);
        return conditionParam;   
    }
	@Override
	public String validateReliveProcess(Map<String, Object> map) throws BusinessException {
		String resultMsg = "";
		boolean result = false;
		Map<String, Object> resultProcess = new HashMap<String, Object>();
    	//流程ID
		String processId= (String)map.get("processId");
		String caseId= (String)map.get("caseId");
		String appName=(String) map.get("appName");
		String summaryId = (String) map.get("summaryId");
		EdocSummary edocSummary = null;
		//1：快速发文和发文拟文都有表单都要验证回写和触发设置
		if(Strings.isNotEmpty(summaryId)){
			edocSummary = govdocSummaryManager.getSummaryById(Long.parseLong(summaryId));
			Integer govdocType = edocSummary.getGovdocType();
			if(govdocType == null)
			{
				return "公文类型不支持流程复活";
			}

			if(govdocType.intValue() == 0)
			{
				return "老公文不支持流程复活";
			}

			//--接下来验证流程的核定节点和 -表单的回调
			if(capFormManager.validateFormFlowCanRelive(edocSummary.getFormAppid())){
				result = true;
			}else{
				result = false;
				resultMsg =  ResourceUtil.getString("supervise.process.resurrection.from.check");
			}

		}else{
			result = false;
			resultMsg ="公文ID不存在";
		}

		if(!result)
		{
			return resultMsg;
		}
		if(edocSummary == null)
		{
			result = false;
			resultMsg ="公文丢失";
		}

		if(!result){
			return resultMsg;
		}
		//2：流程验证
		boolean isQuickSend = edocSummary.getIsQuickSend();

		//3 验证分办
		if(!isQuickSend)
		{
			// 发文拟文验证流程是否可以复活
			if(result) {
				resultProcess = wapi.reliveProcessValidate(appName, processId, caseId);
				String canRelive = (String) resultProcess.get("validateResult");
				String msg = (String) resultProcess.get("msg");
				resultMsg= msg;

				if("true".equals(canRelive)){
					result = true;
				}else{
					result = false;
				}
			}

			if(!result){
				return resultMsg;
			}
			//判断最后一个处理节点是终止还是接受


			if(resultProcess.get("hisWorkitemDaos") != null)
			{
				//如果是流程终止，复活逻辑如下
				//1如果终止，只能是分办
				List<WorkItemInfoBO> hisWorkitemDaos =  (List<WorkItemInfoBO>)resultProcess.get("hisWorkitemDaos");
				for(WorkItemInfoBO workitemInfo: hisWorkitemDaos)
				{
					String chekResult = checkCannReliveProccess(workitemInfo);
					if(chekResult != null)
					{
						return chekResult;
					}

				}

			}
			else
			{
				//2如果是结束 可能是分办可能分送
				//获取最后一个节点
				List<BPMHumenActivity> parentNodes = (List<BPMHumenActivity>) resultProcess.get("parentNodes");
				WorkItemInfoBO workitemInfo= wapi.getLastDoneWorItem(Long.parseLong(caseId), parentNodes);
				String chekResult = checkCannReliveProccess(workitemInfo);
				if(chekResult != null)
				{
					return chekResult;
				}
			}

		}
		else
		{
			if(AppContext.hasPlugin("ocip")&& !OnlineChecker.isOnline()) {
				return "代理没有连接上不能复活";
			}
			//快速发文
			// 不验证流程，没有workitemId
			GovdocExchangeMain exchangeMain = govdocExchangeManager.findBySummaryId(edocSummary.getId(), null);
			if(exchangeMain != null)
			{
				List<GovdocExchangeDetail> listDetail = govdocExchangeManager.getDetailByMainId(exchangeMain.getId());

				if(listDetail !=null && !listDetail.isEmpty())
				{
					for(GovdocExchangeDetail detail : listDetail)
					{
						if(detail.getStatus() == GovdocEnum.ExchangeDetailStatus.waitSign.getKey() || detail.getStatus() == GovdocEnum.ExchangeDetailStatus.sending.getKey())
						{
							return "该数据存在未处理的签收流程，请回退或撤销后再进行流程复活操作。";
						}

					}
				}

			}
			resultMsg = "复活到["+edocSummary.getStartMember().getName()+"]的待发中";
		}

		if(isQuickSend)
		{
			return result + "1" +  resultMsg;
		}
		else {
			return result + "0" +  resultMsg;
		}

	}

	@Override
	public String checkCanReliveForOcip(Long memberId) throws BusinessException {

		if(AppContext.hasPlugin("ocip")&& !OnlineChecker.isOnline()) {
			return "代理没有连接上不能复活";
		}

		if(AppContext.hasPlugin("ocip")&& OnlineChecker.isOnline())
		{

			V3xOrgMember member = orgManager.getMemberById(memberId);
			if(member != null && OrgUtil.isPlatformEntity(member))
			{
				//判断该人员在其他系统是否有人员
				return "处理节点是外系统人员不能恢复";
			}

			if(member != null && !OrgUtil.isPlatformEntity(member) && hasMutiPlugin)
			{
				//查看本地人员在这其他系统人员是否有人
				String sysCode = Global.getSysCode();
				List<String> objectIds = new ArrayList<String>();
				objectIds.add(member.getId().toString());
				Map<String, List<OrgMember>> members = organizationManager.getMemberByObjectIds(objectIds, sysCode);
				if(members != null && !members.isEmpty())
				{
					List<OrgMember> listMember = members.get(member.getId().toString());
					if(listMember != null && listMember.size() > 1)
					{
						return "处理节点是外系统人员不能恢复";
					}

					if(listMember != null && listMember.size() == 1)
					{
						if(!member.getId().toString().equals(listMember.get(0).getObjectId()))
						{
							return "处理节点是外系统人员不能恢复";
						}

					}
				}
			}
		}
		return null;
	}
	private String checkCannReliveProccess(WorkItemInfoBO workitemInfo) throws BusinessException {
		Long wordItemId = workitemInfo.getId();
		List<CtpAffair> affairs = affairManager.getAffairsBySubObjectId(wordItemId);
		for(CtpAffair affair : affairs)
		{
			if(affair.getState() != StateEnum.col_done.key())
			{
				continue;
			}
			String ocipCheckResult = checkCanReliveForOcip(affair.getMemberId());
			if(ocipCheckResult != null)
			{
				return ocipCheckResult;
			}
			GovdocExchangeMain exchangeMain = govdocExchangeManager.findBySummaryId(affair.getObjectId());
			if(exchangeMain != null && affair != null && exchangeMain.getAffairId() != null && affair.getId().longValue() == exchangeMain.getAffairId())
			{
				List<GovdocExchangeDetail> listDetail = govdocExchangeManager.getDetailByMainId(exchangeMain.getId());
				if(listDetail !=null && !listDetail.isEmpty())
				{
					for(GovdocExchangeDetail detail : listDetail)
					{
						if(detail.getStatus() == GovdocEnum.ExchangeDetailStatus.waitSign.getKey() || detail.getStatus() == GovdocEnum.ExchangeDetailStatus.sending.getKey())
						{
							return "该数据存在未处理的签收流程，请回退或撤销后再进行流程复活操作。";
						}

					}
				}
			}
		}

		return null;
	}

	/**
	 * 流程恢复后调用
	 * @param processId
	 */
	@Override
	public void onChangeProcessState(String processId) {



		try {
			EdocSummary edocSummary = govdocSummaryManager.getSummaryByProcessId(processId);
			boolean hasArchive = edocSummary.getHasArchive();
			//1 TODO
			if(edocSummary.getIsQuickSend())
			{
				//如果是快速发文恢复待办重置流程和summary
				govdocManager.processReviveForQuickSend(edocSummary);
			}
			// 2删除交换数据
			GovdocExchangeMain exchangeMain = govdocExchangeManager.findBySummaryId(edocSummary.getId(), null);
			//判断公文收发进行不过交换
			boolean isExchange = false;
			boolean isCurrentNodeFenSong = false;
			boolean isDistributionNodeBefore = false;
			if(exchangeMain != null)
			{
				isExchange = true;
				if(edocSummary.getIsQuickSend())
				{
					isCurrentNodeFenSong = true;
				}
				else
				{
					List<Integer> states = new ArrayList<Integer>();
					states.add( StateEnum.col_pending.key());
					states.add( StateEnum.col_pending_repeat_auto_deal.getKey());
					Map<String,Object> map = new LinkedHashMap<String,Object>();
					map.put("objectId", edocSummary.getId());
					map.put("state",states);
					map.put("delete", false);
					map.put("nodePolicy","zhihui");
					List<CtpAffair> affairList = affairManager.getPengingAffairs(map);
					Long mainAffairId = exchangeMain.getAffairId();
					if(affairList != null)
					{
						for(CtpAffair affair : affairList)
						{
							if(affair.getId().longValue() == mainAffairId)
							{
								//表示当前待办发起过分送
								isCurrentNodeFenSong = true;
								break;
							}
						}
					}
				}


			}

			if(isExchange && !isCurrentNodeFenSong)
			{
				isDistributionNodeBefore = true;
			}

			if(isCurrentNodeFenSong)
			{
				//删除main，删除detail
				govdocExchangeManager.deleteDetailByMainId(exchangeMain.getId());
//				govdocExchangeManager.deleteMainBySummaryId(edocSummary.getId());
			}

			//3处理文号
			govdocMarkManager.dealMark4ReviveEdoc(edocSummary,isDistributionNodeBefore);
			//4统计
			if(edocSummary.getIsQuickSend())
			{
				//删除公文统计的对应summary数据
				edocStatManager.deleteEdocStat(edocSummary.getId());

				//修改动态表
				FormBean formBean = govdocFormManager.getForm(edocSummary.getFormAppid());
				String tableName = formBean.getMasterTableBean().getTableName();
				String sql ="update "+ tableName +" set ";
				sql += "state=?,finishedflag=?";
				List<Integer> params = new ArrayList<Integer>();
				params.add(0);
				params.add(0);


				sql +=" where id=" + edocSummary.getFormRecordid();

				JDBCAgent jdbcAgent = new JDBCAgent();
				try {
					jdbcAgent.execute(sql, params);
				} catch (Exception e) {
					LOGGER.error("快速发文复活修改动态表失败",e);
				}finally {
					if (jdbcAgent != null) {
						jdbcAgent.close();
					}
				}

				//删除归档
				if (hasArchive && AppContext.hasPlugin("doc")) {
					List<Long> sourceId = new ArrayList<Long>();
					sourceId.add(exchangeMain.getAffairId());
					docApi.deleteDocResources(AppContext.getCurrentUser().getId(),sourceId);
				}
			}
			else
			{
				//修改动态表
				FormBean formBean = govdocFormManager.getForm(edocSummary.getFormAppid());
				String tableName = formBean.getMasterTableBean().getTableName();
				String sql ="update "+ tableName +" set ";
				sql += "state=?,finishedflag=?";
				List<Integer> params = new ArrayList<Integer>();
				params.add(1);
				params.add(0);


				sql +=" where id=" + edocSummary.getFormRecordid();

				JDBCAgent jdbcAgent = new JDBCAgent();
				try {
					jdbcAgent.execute(sql, params);
				} catch (Exception e) {
					LOGGER.error("快速发文复活修改动态表失败",e);
				}finally {
					if (jdbcAgent != null) {
						jdbcAgent.close();
					}
				}
				edocStatManager.updateFlowState(edocSummary.getId(),edocSummary.getState());
			}

		} catch (Exception e) {
		}



	}
	
	public String getMyManagerTemplateCount() throws BusinessException{
	    return "0";
	}

	@Override
	public String updateAppointStepBack(Long currentAffairId, String stepBackNodeId, String submitType, String comment)
			throws BusinessException {
		GovdocDealVO dealVo = new GovdocDealVO();
		dealVo.setCurrentUser(AppContext.getCurrentUser());
		CtpAffair affair  = affairManager.get(currentAffairId);
		//需要判断是否有分送过
		if(govdocExchangeManager.findBySummaryId(affair.getObjectId(), GovdocExchangeMain.EXCHANGE_TYPE_JIAOHUAN) != null){
			return ResourceUtil.getString("govdoc.rollback.distribute.canNot");
		}
		dealVo.setSummary(govdocSummaryManager.getSummaryById(affair.getObjectId()));
		dealVo.setAffair(affair);
		dealVo.setSummaryId(affair.getObjectId());
		dealVo.setAffairId(affair.getId());
		dealVo.setWorkitemId(affair.getSubObjectId());
		dealVo.setActivityId(affair.getActivityId());
		dealVo.setCaseId(affair.getCaseId());
		dealVo.setProcessId(affair.getProcessId());
		dealVo.setSelectTargetNodeId(stepBackNodeId);
		dealVo.setSubmitStyle(submitType);
		dealVo.setIsWFTrace("1");//默认追溯
		dealVo.setIsCircleBack("false");
		WorkflowTraceEnums.workflowTrackType trackType = WorkflowTraceEnums.workflowTrackType.special_step_back_repeal;
		if ("1".equals(dealVo.getIsCircleBack())) {
			trackType = WorkflowTraceEnums.workflowTrackType.circle_step_back_repeal;
		}
		dealVo.setTrackWorkflowType(String.valueOf(trackType.getKey()));
		Comment comment1 = GovdocUtil.createNullCommentWithoutAttitude(affair, affair.getObjectId());
		comment1.setId(null);//指定回退沒有意见
		comment1.setContent(comment);
		if(dealVo.getCurrentUser().isAdmin())
        {
            comment1.setExtAtt1("govdoc.administrator.operation");
            comment1.setExtAtt3("collaboration.dealAttitude.rollback");
        }
		dealVo.setComment(comment1);
		govdocManager.transAppointStepBack(dealVo);
		return null;
	}

	@Override
	public String replaceWorkitem(CtpAffair affair, V3xOrgMember replaceMember, User currentUser)
			throws BusinessException {
		Map<String,String> params = new HashMap<String,String>();
    	params.put("affairId", affair.getId()+"");
    	params.put("transferMemberId", replaceMember.getId()+"");
    	params.put("fromBatch", "1");
    	boolean isLock = false;
    	try {
        	isLock = govdocLockManager.canGetLock(affair.getId());
        	if (!isLock) {
        		return ResourceUtil.getString("govdoc.not.getlock");
        	}
        	String errorMsg = this.govdocManager.transfer(params);
        	if(Strings.isNotBlank(errorMsg)) {
        		return errorMsg;
        	}
		} catch(Exception e) {
			return ResourceUtil.getString("govdoc.alert.message.transfer.error");
		} finally {
			if(isLock) {
				govdocLockManager.unlock(affair.getId());
			}
		}
		return null;
	}

    
	public void setAffairManager(AffairManager affairManager) {
		this.affairManager = affairManager;
	}
	public void setGovdocManager(GovdocManager govdocManager) {
		this.govdocManager = govdocManager;
	}
	public void setGovdocListManager(GovdocListManager govdocListManager) {
		this.govdocListManager = govdocListManager;
	}
	public void setGovdocSummaryManager(GovdocSummaryManager govdocSummaryManager) {
		this.govdocSummaryManager = govdocSummaryManager;
	}
	public void setGovdocLockManager(GovdocLockManager govdocLockManager) {
		this.govdocLockManager = govdocLockManager;
	}
	public void setPermissionManager(PermissionManager permissionManager) {
		this.permissionManager = permissionManager;
	}
	public void setGovdocPubManager(GovdocPubManager govdocPubManager) {
		this.govdocPubManager = govdocPubManager;
	}

	public void setGovdocExchangeManager(GovdocExchangeManager govdocExchangeManager) {
		this.govdocExchangeManager = govdocExchangeManager;
	}

	public void setCapFormManager(CAPFormManager capFormManager) {
		this.capFormManager = capFormManager;
	}

	public void setWapi(WorkflowApiManager wapi) {
		this.wapi = wapi;
	}

	public void setOrgManager(OrgManager orgManager) {
		this.orgManager = orgManager;
	}

	public void setOrganizationManager(IOrganizationManager organizationManager) {
		this.organizationManager = organizationManager;
	}

	public void setEdocStatManager(EdocStatManager edocStatManager) {
		this.edocStatManager = edocStatManager;
	}

	public void setGovdocMarkManager(GovdocMarkManager govdocMarkManager) {
		this.govdocMarkManager = govdocMarkManager;
	}
	public void setDocApi(DocApi docApi) {
		this.docApi = docApi;
	}

	public void setGovdocFormManager(GovdocFormManager govdocFormManager) {
		this.govdocFormManager = govdocFormManager;
	}
}
