package com.seeyon.apps.govdoc.manager.impl;

import static java.io.File.separator;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.seeyon.apps.collaboration.vo.AttachmentVO;
import com.seeyon.apps.doc.api.DocApi;
import com.seeyon.apps.edoc.constants.EdocConstant;
import com.seeyon.apps.edoc.constants.EdocConstant.SendType;
import com.seeyon.apps.govdoc.constant.GovdocAppLogAction;
import com.seeyon.apps.govdoc.helper.GovdocContentHelper;
import com.seeyon.apps.govdoc.manager.GovdocCommentManager;
import com.seeyon.apps.govdoc.manager.GovdocContentManager;
import com.seeyon.apps.govdoc.manager.GovdocLogManager;
import com.seeyon.apps.govdoc.manager.GovdocMessageManager;
import com.seeyon.apps.govdoc.manager.GovdocSignetManager;
import com.seeyon.apps.govdoc.manager.GovdocSummaryManager;
import com.seeyon.apps.govdoc.util.GovdocUtil;
import com.seeyon.apps.govdoc.vo.GovdocAttachmentVO;
import com.seeyon.apps.govdoc.vo.GovdocAttachmentVO.FromType;
import com.seeyon.apps.govdoc.vo.GovdocBaseVO;
import com.seeyon.apps.govdoc.vo.GovdocBodyVO;
import com.seeyon.apps.govdoc.vo.GovdocNewVO;
import com.seeyon.apps.govdoc.vo.GovdocSummaryVO;
import com.seeyon.apps.index.api.IndexApi;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.GlobalNames;
import com.seeyon.ctp.common.ModuleType;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.affair.util.AffairUtil;
import com.seeyon.ctp.common.affair.util.AttachmentEditUtil;
import com.seeyon.ctp.common.appLog.AppLogAction;
import com.seeyon.ctp.common.appLog.manager.AppLogManager;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.constants.SystemProperties;
import com.seeyon.ctp.common.content.ContentConfig;
import com.seeyon.ctp.common.content.ContentViewRet;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.content.affair.constants.SubStateEnum;
import com.seeyon.ctp.common.content.comment.Comment;
import com.seeyon.ctp.common.content.mainbody.MainbodyManager;
import com.seeyon.ctp.common.content.mainbody.MainbodyService;
import com.seeyon.ctp.common.content.mainbody.MainbodyType;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.filemanager.Constants;
import com.seeyon.ctp.common.filemanager.dao.AttachmentDAO;
import com.seeyon.ctp.common.filemanager.domain.ReplaceBase64Result;
import com.seeyon.ctp.common.filemanager.manager.AttachmentManager;
import com.seeyon.ctp.common.filemanager.manager.FileManager;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.office.manager.OfficeBakFileManager;
import com.seeyon.ctp.common.office.trans.manager.OfficeTransManager;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.po.content.CtpContentAll;
import com.seeyon.ctp.common.po.filemanager.Attachment;
import com.seeyon.ctp.common.po.filemanager.V3XFile;
import com.seeyon.ctp.common.processlog.ProcessLogAction;
import com.seeyon.ctp.common.processlog.manager.ProcessLogManager;
import com.seeyon.ctp.common.processlog.po.ProcessLog;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.Datetimes;
import com.seeyon.ctp.util.ParamUtil;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.UUIDLong;
import com.seeyon.ctp.util.json.JSONUtil;
import com.seeyon.ctp.workflow.wapi.WorkflowApiManager;
import com.seeyon.v3x.edoc.domain.EdocSummary;

public class GovdocContentManagerImpl implements GovdocContentManager {

	private Log LOGGER = LogFactory.getLog(GovdocManagerImpl.class);
	
	private GovdocSummaryManager govdocSummaryManager;
	private GovdocCommentManager govdocCommentManager;
	private GovdocSignetManager govdocSignetManager;
	private GovdocLogManager govdocLogManager;
	private GovdocMessageManager govdocMessageManager;
	private WorkflowApiManager wapi;
	private AffairManager affairManager;
	private OrgManager orgManager;
	private MainbodyManager ctpMainbodyManager;
	private IndexApi indexApi;
	private FileManager fileManager;
	private AttachmentManager attachmentManager;
	private AttachmentDAO attachmentDAO;
	private DocApi docApi;
    private ProcessLogManager            processLogManager;
    private AppLogManager                appLogManager;
	private OfficeBakFileManager officeBakFileManager;
    private OfficeTransManager officeTransManager;
	
	/*************************** 00000 公文拟文正文 start ***************************/
	@Override
	public void fillNewGovdocTransByAtt(GovdocNewVO newVo, List<Attachment> atts) throws BusinessException {
		String attListJSON = attachmentManager.getAttListJSON(atts);
		newVo.setAttListJSON(attListJSON);
		//循环附件，过滤出关联文档。附件说明用到
		List<Attachment> contentAtts = new ArrayList<Attachment>();
		for (Attachment attachment : atts) {
			if(attachment.getType() != 2){
				contentAtts.add(attachment);
			}
		}
		String contentFilesmAttName = attachmentManager.getFileNameExcludeSuffix(contentAtts);
		newVo.setFilesmContentAttsName(contentFilesmAttName);
	}

	/*************************** 00000 公文拟文正文     end ***************************/
	
	/*************************** 11111 公文正文 start ***************************/
	@Override
	public ContentViewRet contentViewForDetail(GovdocSummaryVO summaryVO) throws BusinessException {
		Long summaryId = -1L;
		if (summaryVO.getSummary() != null) {
			summaryId = summaryVO.getSummary().getId();
		}
		summaryVO.setSummaryId(summaryId);
		
		HttpServletRequest request = (HttpServletRequest) AppContext.getThreadContext(GlobalNames.THREAD_CONTEXT_REQUEST_KEY);

		
		
		ContentConfig contentCfg = ContentConfig.getConfig(ModuleType.edoc);

		List<StateEnum> states = new ArrayList<StateEnum>();
		states.add(StateEnum.col_sent);
		states.add(StateEnum.col_done);
		states.add(StateEnum.col_pending);
		states.add(StateEnum.col_waitSend);
		List<CtpAffair> pushMessageList = affairManager.getAffairs(summaryId, states);
			
		// 排序顺序规则，发起人、已办、暂存待办
		Collections.sort(pushMessageList, new Comparator<CtpAffair>() {
			@Override
			public int compare(CtpAffair o1, CtpAffair o2) {
				if (o1.getState() == StateEnum.col_sent.getKey())
					return -1;
				else if (o2.getState() == StateEnum.col_sent.getKey())
					return 1;
				else {
					if (o1.getState() == StateEnum.col_done.key())
						return -1;
					else if (o2.getState() == StateEnum.col_done.key())
						return 1;
					else
						return 0;
				}
			}
		});
		
		// 过滤掉自己和重复项
		Map<Long, Boolean> memberIdMap = new HashMap<Long, Boolean>(pushMessageList.size());
		// 消息推送
		List<CtpAffair> pushMessageListAffair = new ArrayList<CtpAffair>();
		Long currentUserId = AppContext.currentUserId();
		for (CtpAffair r : pushMessageList) {
			// 只显示已发、暂存待办和、已办的、回退者
			int subState = r.getSubState();
			int state = r.getState();
			if ((subState == SubStateEnum.col_pending_ZCDB.key() && state == StateEnum.col_pending.key()) || state == StateEnum.col_done.key()
					|| state == StateEnum.col_sent.key() || subState == SubStateEnum.col_pending_specialBack.key()
					|| subState == SubStateEnum.col_pending_specialBacked.key() || subState == SubStateEnum.col_pending_specialBackCenter.key()
					|| state == StateEnum.col_pending.key()) {
				Long memberId = r.getMemberId();
				if (!memberId.equals(currentUserId) && memberIdMap.get(memberId) == null) {
					memberIdMap.put(memberId, Boolean.TRUE);
					pushMessageListAffair.add(r);
				}
			}
		}
		
		request.setAttribute("commentPushMessageToMembersList", pushMessageListAffair);
		request.setAttribute("__huanhang", "\r\n");
		
		request.setAttribute("contentCfg", contentCfg);
		
		
		ContentViewRet context = new ContentViewRet();
		context.setModuleId(summaryId);
		context.setModuleType(ModuleType.edoc.getKey());
		
		Long affairId = summaryVO.getAffairId();
		context.setAffairId(affairId);
		request.setAttribute("contentContext", context);
		return context;
	}
	
	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	public void fillSummaryVoByAtt(GovdocSummaryVO summaryVO) throws BusinessException {
		// 正文区域附件
		String arrListJSON = "";
		List<Attachment> mainAtt = new ArrayList<Attachment>();
		//表单上的附件
		Set<Attachment> formAtt = new HashSet<Attachment>();
		List<Attachment> formList = new ArrayList<Attachment>();
		
		//流转附件
		List<Attachment> commentShowAttrs = new ArrayList<Attachment>();
		// 显示保存了草稿的意见、附件、意见隐藏不包括人
		List<Attachment> allAttachments = new ArrayList<Attachment>();
		
		if(summaryVO.getSummary().getNewflowType()==EdocConstant.NewflowType.child.ordinal()){
			Long mainProcessId = wapi.getMainProcessIdBySubProcessId(Long.parseLong(summaryVO.getProcessId()));
			if(mainProcessId != null){
				EdocSummary mainSummary = govdocSummaryManager.getSummaryByProcessId(mainProcessId.toString());
				List<Attachment> atts = attachmentManager.getByReference(mainSummary.getId());
				for (Attachment attachment : atts) {
					if(attachment.getCategory()==ApplicationCategoryEnum.form.getKey()){
						formAtt.add(attachment);
					}
				}
			}
		}else{
			if(Strings.isNotBlank(summaryVO.getProcessId())){
				List<Long> subProcessIds = wapi.getSubProcessRunningIdsByMainProcessId(Long.parseLong(summaryVO.getProcessId()));
				if(CollectionUtils.isNotEmpty(subProcessIds)){
					for (Long subProcessId : subProcessIds) {
						EdocSummary mainSummary = govdocSummaryManager.getSummaryByProcessId(subProcessId.toString());
						List<Attachment> atts = attachmentManager.getByReference(mainSummary.getId());
						for (Attachment attachment : atts) {
							if(attachment.getCategory()==ApplicationCategoryEnum.form.getKey()){
								formAtt.add(attachment);
							}
						}
					}
				}
			}
		}
		
		List<Attachment> temp = attachmentManager.getByReference(summaryVO.getSummaryId());
		// add by rz 2010-11-11 [修改bug附件显示顺序不正确] start
		if (!temp.isEmpty()) {
			Collections.sort(temp, new Comparator<Object>() {
				@Override
				public int compare(Object o1, Object o2) {
					Attachment stu1 = (Attachment) o1;
					Attachment stu2 = (Attachment) o2;
					int compareResult =  Long.valueOf(stu1.getCreatedate().getTime()).compareTo(Long.valueOf(stu2.getCreatedate().getTime()));

					if(compareResult == 0)
					{
						return Integer.valueOf(stu1.getSort()).compareTo(Integer.valueOf(stu2.getSort()));
					}
					return compareResult;


				}
			});
			allAttachments.addAll(temp);
		}
		comtentDraftAttAndDis(allAttachments, summaryVO);
		if (Strings.isNotEmpty(allAttachments)) {
			for (Attachment att : allAttachments) {
				if(summaryVO.getSummaryId().equals(att.getSubReference())
						&& (Integer.valueOf(com.seeyon.ctp.common.filemanager.Constants.ATTACHMENT_TYPE.FILE.ordinal()).equals(att.getType()) 
								|| Integer.valueOf(com.seeyon.ctp.common.filemanager.Constants.ATTACHMENT_TYPE.DOCUMENT.ordinal()).equals(att.getType()))) {
						mainAtt.add(att);
				}else if(att.getCategory() == ApplicationCategoryEnum.form.getKey()){
					formAtt.add(att);
				}else{
					commentShowAttrs.add(att);
				}
			}
		}
		// 区分老公文还是新公文 然后设置类型 让关联文档正常打开
		for (Attachment att : mainAtt) {
			if (att.getFileUrl() != null) {
				CtpAffair aff = affairManager.get(att.getFileUrl());
				if (aff != null && aff.getApp() > 4) {
					att.setMimeType("edoc");
				}
			}
		}
		//过滤关联文档,附件说明需要使用纯附件
		List<Attachment> newMainAtt = new ArrayList<Attachment>();
		for(Attachment attachment:mainAtt){
			if(Integer.valueOf(Constants.ATTACHMENT_TYPE.FILE.ordinal()).equals(attachment.getType())){
				newMainAtt.add(attachment);
			}
		}
		//一屏式附件
		if(!"0".equals(summaryVO.getSwitchVo().getNewGovdocView())){
			int attNum = 0;
	        List<Attachment> setAtt = new ArrayList<Attachment>();
			mainAtt.addAll(formAtt);
			for(Attachment a:mainAtt){
				try{
					Attachment clone = (Attachment) BeanUtils.cloneBean(a);
					clone.setSubReference(a.getReference());
					setAtt.add(clone);
				}catch(Exception e){
					
				}
			}
			mainAtt.clear();
			mainAtt.addAll(setAtt);
			List<Attachment> commentAttList = new ArrayList<Attachment>();
			List<Comment> commentList = govdocCommentManager.getCommentAllByModuleId(ModuleType.edoc, summaryVO.getSummary().getId());
			for (Comment comment : commentList) {
				if(comment.getCtype()==Comment.CommentType.draft.getKey()||
						comment.getCtype()==Comment.CommentType.govdocniban.getKey()){//不显示存为模板附件
					continue;
				}
				String atts = comment.getRelateInfo();
	            if(Strings.isNotBlank(atts) && atts.indexOf(":") != -1){
	                try{
	                    List list = JSONUtil.parseJSONString(atts,List.class);
	                    List<Attachment> l = ParamUtil.mapsToBeans(list, Attachment.class, false);
	                    
	                    commentAttList.addAll(l);
	                }catch(Exception e){
	                	LOGGER.error(e);
	                }
	            }
	            //add by rz 2017-09-09 [处理代录数据] start
	     		String pishiComments=comment.getContent();
	     		if(StringUtils.isNotBlank(pishiComments)){
		     		if(pishiComments.indexOf(ResourceUtil.getString("govdoc.generation.record"))!=-1){
		     			comment.setExtAtt2("pishi");
		     		}
	     		}
	            //add by rz 2017-09-09 [处理代录数据] end
			}
			List<Attachment> commentAtt = new ArrayList<Attachment>();
			for(Attachment a:commentAttList){
				try{
					Attachment clone = (Attachment) BeanUtils.cloneBean(a);
					clone.setSubReference(a.getReference());
					commentAtt.add(clone);
				}catch(Exception e){
					
				}
			}
			commentAttList.clear();
			commentAttList.addAll(commentAtt);
			String commentShowAttrstr = attachmentManager.getAttListJSON(commentAttList);
			AppContext.putRequestContext("commentShowAttrstr",commentShowAttrstr);
			if(mainAtt!=null&&commentShowAttrs!=null){
				attNum = mainAtt.size()+commentAttList.size();
			}
			AppContext.putRequestContext("attNum",attNum);
		}
		arrListJSON = attachmentManager.getAttListJSON(mainAtt);
		AppContext.putRequestContext("attListJSON", arrListJSON);
		formList.addAll(formAtt);
		//表单中附件的文件名(附件说明用到)
		String formAttsName = attachmentManager.getFileNameExcludeSuffix(formList);
		AppContext.putRequestContext("formAttsName", formAttsName);
		//正文区附件的文件名(附件说明用到)
		String contentAttsName = attachmentManager.getFileNameExcludeSuffix(newMainAtt);
		AppContext.putRequestContext("contentAttsName", contentAttsName);
	}
	/**
	 * 显示保存了草稿的意见、附件、意见隐藏不包括人
	 *
	 * @param attachments
	 * @param affair
	 */
	private void comtentDraftAttAndDis(List<Attachment> attachments, GovdocSummaryVO summaryVO) {
		CtpAffair affair = summaryVO.getAffair();
		// 处理区域附件
		String handleAttach = "";
		Long commentId = null;
		List<Attachment> dealAtt = new ArrayList<Attachment>();

		// 获取草稿
		Comment comentDraft = govdocCommentManager.getDraftOpinion(affair.getId());
		//客开 项目名称： [修改功能：] 作者：fzc 修改日期：2018-4-28 start
		String attitude =(String) AppContext.getRequestContext("nodeattitude");
		if(Strings.isNotBlank(attitude)&&comentDraft!=null) {
			String[] strs = attitude.split(",");
			boolean hasAttitude = false;
			for(String s :strs) {
				if(s.equals(comentDraft.getExtAtt4())) {
					hasAttitude = true;
					break;
				}
			}
			if(!hasAttitude) {
				comentDraft.setExtAtt4(null);
				comentDraft.setExtAtt1(null);
			}

		}
 		AppContext.putRequestContext("commentDraft", comentDraft);
 		//客开 项目名称： [修改功能：] 作者：fzc 修改日期：2018-4-28 end
		// 获取意见隐藏的ids
		String displayIds = "";
		// 获取意见隐藏的Names
		String displayNames = "";
		try {
			if (affair.getState().equals(StateEnum.col_pending.key())) {
				if (comentDraft != null) {
					commentId = comentDraft.getId();
					for (Attachment att : attachments) {
						if (commentId != null && commentId.equals(att.getSubReference())) {
							dealAtt.add(att);
						}
					}
					// 意见隐藏
					if (comentDraft.getShowToId() != null) {
						String[] ids = comentDraft.getShowToId().split(",");
						if (!"".equals(ids[0])) {
							for (int i = 0; i < ids.length; i++) {
								V3xOrgMember mem = orgManager.getMemberById(Long.valueOf(ids[i]));
								if (i == ids.length - 1) {
									displayIds = displayIds + "Member|" + ids[i];
									displayNames = displayNames + mem.getName();
								} else {
									displayIds = displayIds + "Member|" + ids[i] + ",";
									displayNames = displayNames + mem.getName() + ",";
								}
							}
						}
					}
				}
			} else {
				// 设置一个标识，用于页面的附件区域
				commentId = 20L;
			}
		} catch (Exception e) {
			LOGGER.error("获取草稿中的附件、意见隐藏人报错！", e);
		}

		summaryVO.setDisplayIds(displayIds);
		summaryVO.setDisplayNames(displayNames);
		
		handleAttach = attachmentManager.getAttListJSON(dealAtt);
		AppContext.putRequestContext("commentId", commentId);
		AppContext.putRequestContext("handleAttachJSON", handleAttach);
	}
	/*************************** 11111 公文正文查看   end ***************************/
	
	
	/*************************** 22222 公文正文保存 start ***************************/
	@Override
	public void saveBodyContent(GovdocNewVO newVo) throws BusinessException {
		if(newVo.getBodyVo().getContent() == null) {
			return;
		}
		//不需要克隆
		CtpContentAll newContentAll = GovdocContentHelper.getBodyContentByModuleId(newVo.getSummary().getId(), false);
		if (null == newContentAll) {
			newContentAll = new CtpContentAll();
			newContentAll.setIdIfNew();
		} 
		newContentAll.setModifyDate(new Date());
		newContentAll.setModuleId(newVo.getSummary().getId());
		newContentAll.setCreateId(newVo.getCurrentUser().getId());
		newContentAll.setContentTemplateId(0L);
		newContentAll.setModuleTemplateId(0L);
		newContentAll.setCreateDate(new Date());
		newContentAll.setContent(String.valueOf(newVo.getBodyVo().getContent()));
		newContentAll.setSort(1);
		newContentAll.setModuleType(ApplicationCategoryEnum.edoc.getKey()); // 公文类型
		newContentAll.setContentType(newVo.getBodyVo().getContentType());
		newContentAll.setModifyId(newVo.getCurrentUser().getId());

		ReplaceBase64Result result = fileManager.replaceBase64Image(newContentAll.getContent());
		newContentAll.setContent(result.getHtml());

		ctpMainbodyManager.saveOrUpdateContentAll(newContentAll);
	}
	@Override
	public void saveBodyContentByFenban(GovdocNewVO newVo) throws BusinessException {
		if (newVo.getSignSummaryId() != null) {
			List<CtpContentAll> ctpContentAlls = GovdocContentHelper.getBodyContentListByModuleId(newVo.getSignSummaryId());
			for (CtpContentAll ctpContentAll : ctpContentAlls) {
				if (ctpContentAll.getContentType() != MainbodyType.HTML.getKey()) {// 如果不是标准正文
					Long fileId = Long.parseLong(ctpContentAll.getContent());
					Long newFileId = fileManager.copyFileBeforeModify(fileId);
					if (newFileId != -1L) {
						ctpContentAll.setContent(String.valueOf(newFileId));
						// 复制签章
						govdocSignetManager.copySignet(fileId, newFileId);
					}
				}
				try {
					CtpContentAll newConetent = (CtpContentAll) ctpContentAll.clone();
					newConetent.setId(UUIDLong.longUUID());
					newConetent.setModuleId(newVo.getSummaryId());
					newConetent.setCreateDate(new Date());
					newConetent.setModifyDate(new Date());
					ctpMainbodyManager.saveOrUpdateContentAll(newConetent);
				} catch(Exception e) {
					LOGGER.error("触发公文时生成公文正文数据出错：", e);
				}
			}
			
			// 获取附件
			List<Attachment> attachments = attachmentManager.getByReference(newVo.getSignSummaryId());
			List<Attachment> attachs = new ArrayList<Attachment>();
			if (attachments != null) {
				for (Attachment attachment : attachments) {
					if (attachment.getType() != Constants.ATTACHMENT_TYPE.FILE.ordinal()) {
						continue;
					}
					if (attachment.getCategory() == ApplicationCategoryEnum.edoc.getKey()) {
						String srcPath = fileManager.getFolder(attachment.getCreatedate(), true) + separator + String.valueOf(attachment.getFileUrl());
						File srcFile = new File(srcPath);
						if (srcFile.exists() && srcFile.isFile()) {
							try {
								InputStream in = new FileInputStream(srcFile);	
								V3XFile f = fileManager.save(in, ApplicationCategoryEnum.edoc, attachment.getFilename(), attachment.getCreatedate(), false);
								Attachment cAtt = new Attachment(f, ApplicationCategoryEnum.edoc, com.seeyon.ctp.common.filemanager.Constants.ATTACHMENT_TYPE.FILE);
								attachs.add(cAtt);
							} catch(FileNotFoundException infe) {
								LOGGER.error("分办的附件文件未找到：", infe);
								throw new BusinessException(infe);
							}
						} else {
							LOGGER.error("分办的附件文件不存在：" + srcFile);
						}
					}
				}
				newVo.setFenbanFile(1);
				String attListJSON = attachmentManager.getAttListJSON(attachs);
				newVo.setAttListJSON(attListJSON);
			}
		}
	}
    @Override
    public void saveExchangeFormContent(GovdocNewVO newVo) throws BusinessException {
    	CtpContentAll formContent = newVo.getBodyVo().getFormContent();
        if(formContent != null) {
        	formContent.setModuleId(newVo.getSummary().getId());
        	formContent.setCreateId(newVo.getCurrentUser().getId());
        	formContent.setContentTemplateId(newVo.getSummary().getFormAppid());
        	formContent.setModuleTemplateId(newVo.getSummary().getTempleteId());
        	formContent.setTitle(newVo.getSummary().getSubject());
            ctpMainbodyManager.saveOrUpdateContentAll(formContent);
        }
    }
    @Override
    public void saveExchangeBodyContent(GovdocNewVO newVo) throws BusinessException {
    	CtpContentAll bodyContent = newVo.getBodyVo().getBodyContent();
        if(bodyContent != null) {
        	bodyContent.setNewId();
        	bodyContent.setTitle(null);
        	bodyContent.setContentDataId(null);
        	bodyContent.setContentTemplateId(0L);
        	bodyContent.setModuleTemplateId(newVo.getSummary().getTempleteId());
        	bodyContent.setCreateId(newVo.getCurrentUser().getId());
        	bodyContent.setModuleId(newVo.getSummary().getId());
//        	bodyContent.setTransId(null);
            ctpMainbodyManager.saveOrUpdateContentAll(bodyContent);
        }
    }
	/*************************** 22222 公文正文保存   end ***************************/
	
	
	/*************************** 33333 公文附件查看 start ***************************/
	@SuppressWarnings("rawtypes")
	@Override
	public List<AttachmentVO> getAttachmentListBySummaryId(Long summaryId, Long memberId) throws BusinessException {
		List<Attachment> tempattachments = null;
		List<Attachment> attachments = new ArrayList<Attachment>();
		List<AttachmentVO> attachmentVOs = new ArrayList<AttachmentVO>();
		// 收藏时候传入的文件Id
		List<Long> attmentIds = new ArrayList<Long>();
		AttachmentVO vo = null;
		HttpServletRequest request = AppContext.getRawRequest();
		String formAttrId = request.getParameter("formAttrId");
		List formAttrIds = new ArrayList<String>();
		// 获取正文组件中自带的附件
		String attmentContent = request.getParameter("attmentList");
		List<String> attmentList = new ArrayList<String>();
		List<String> _attmentList = new ArrayList<String>();
		if (Strings.isNotBlank(attmentContent) && !"null".equals(attmentContent)) {
			_attmentList = Arrays.asList(attmentContent.split(","));
			for (int a = _attmentList.size() - 1; a > -1; a--) {
				if (!attmentList.contains(_attmentList.get(a))) {
					attmentList.add(_attmentList.get(a));
				}
			}
		}
		if (Strings.isNotBlank(formAttrId)) {
			formAttrIds = Arrays.asList(formAttrId.split(","));
		}
		EdocSummary colSummary = govdocSummaryManager.getSummaryById(summaryId);
		tempattachments = attachmentManager.getByReference(summaryId);
		
		//附件列表显示顺序按照sort显示
		if (!tempattachments.isEmpty()) {
			Collections.sort(tempattachments, new Comparator<Object>() {
				@Override
				public int compare(Object o1, Object o2) {
					Attachment stu1 = (Attachment) o1;
					Attachment stu2 = (Attachment) o2;
					return Integer.valueOf(stu1.getSort()).compareTo(Integer.valueOf(stu2.getSort()));
				}
			});
		}

		// 控制隐藏的评论对发起人可见
		AppContext.putThreadContext(Comment.THREAD_CTX_NOT_HIDE_TO_ID_KEY, colSummary.getStartMemberId());

		// 添加附件到对象中，附件的type为0，关联文档的type为2（不显示关联文档在附件列表中）
		for (Attachment attachment : tempattachments) {
			if (attachment.getType() == 0) {
				attachments.add(attachment);
			}
		}
		if (attachments != null && attachments.size() > 0) {
			boolean isHistoryFlag = "true".equals(request.getParameter("isHistoryFlag")) ? true : false;
			List<Comment> comments = govdocCommentManager.getCommentAllByModuleId(ModuleType.edoc, summaryId, isHistoryFlag);
			//List<Comment> childComments = new ArrayList<Comment>();
			for (Attachment attachment : attachments) {
				vo = new AttachmentVO();
				GovdocContentHelper.createAttachmentVO(vo, attachment);
				// 创建人
				Long commentId = attachment.getSubReference();
				Comment curComment = null;
				for (Comment comment : comments) {
					if (comment.getId().longValue() == commentId.longValue()) {
						curComment = comment;
						break;
					}
				}
				/**
				 * 1、发起人上传：标题下方的由发起人上传的附件，如果流程中间的节点跟发起人是同一个人，也算”处理人回复“
				 * 2、处理人回复：意见区域的附件（评论&回复），通过修改附件区域上传的附件 3、表单控件：表单正文里面的附件
				 * 4、附言补充：发起人附言区域里面的附件
				 */

				if (curComment != null) {
					if (!curComment.isCanView())
						continue;
					String agentName = curComment.getExtAtt2();
					Long createId = curComment.getCreateId();
					if (Strings.isNotBlank(agentName)) {
						vo.setUserName(agentName);
					} else {
						V3xOrgMember member = orgManager.getMemberById(createId);
						vo.setUserName(member.getName());
					}

					String fromType = "";
					// 来源
					Integer cType = curComment.getCtype();
					// 如果是草稿状态时，不显示附件
					if (cType != null) {
						int ct = cType.intValue();
						if (ct == -2) {
							continue;
						}
						if (curComment.getCtype().intValue() == Comment.CommentType.govdocniban.getKey())// 如果是拟办意见的，不显示
							continue;
						// 如果是转发的则显示成正文区
						if (curComment.getForwardCount() > 0) {
							fromType = ResourceUtil.getString("collaboration.att.form");
						} else {
							if (ct == -1) {
								fromType = ResourceUtil.getString("collaboration.att.sender");// "附言区";
							} else if (ct == 1 || ct == 0) {
								fromType = ResourceUtil.getString("collaboration.att.reply");// "处理区";
							}
						}
					}
					attmentIds.add(attachment.getFileUrl());
					vo.setFromType(fromType);
					attachmentVOs.add(vo);
				} else if (summaryId.equals(attachment.getSubReference())) {
					// 标题区
					Date attaDate = attachment.getCreatedate();
					Date colDate = colSummary.getCreateTime();
					// 将协同创建时间加上2秒,以判断是否是发起人上传，因为在发起人上传时，必须保证附件的创建时间小于协同的创建时间
					// 但是在实际情况中由于保存时都是用的new Date()方式存储日期，导致时间有误差(ms级)
					if (attaDate.getTime() < colDate.getTime() + 2000) {
						try {
							V3xOrgMember member = orgManager.getMemberById(colSummary.getStartMemberId());
							vo.setUserName(member.getName());
						} catch (Exception e) {
							LOGGER.error("", e);
						}
						vo.setFromType(ResourceUtil.getString("collaboration.att.titleArea"));// 标题区
					} else {
						V3XFile file = fileManager.getV3XFile(attachment.getFileUrl());
						String name = "";
						if (file != null) {
							try {
								V3xOrgMember member = orgManager.getMemberById(file.getCreateMember());
								name = member.getName();
							} catch (Exception e) {
								LOGGER.error("", e);
							}
						}
						vo.setUserName(name);
						// 处理人回复（处理区）
						vo.setFromType(ResourceUtil.getString("collaboration.att.titleArea"));
					}
					attmentIds.add(attachment.getFileUrl());
					attachmentVOs.add(vo);
				} else if (AttachmentEditUtil.CONTENT_ATTACHMENTSUBRE.equals(attachment.getSubReference().toString())) { // 来之转发表单控件的附件
					V3XFile file = fileManager.getV3XFile(attachment.getFileUrl());
					String name = "";
					if (file != null) {
						try {
							V3xOrgMember member = orgManager.getMemberById(file.getCreateMember());
							name = member.getName();
						} catch (Exception e) {
							LOGGER.error("", e);
						}
					}
					vo.setUserName(name);
					// （来自表单转发控件的属于正文区域）
					vo.setFromType(ResourceUtil.getString("collaboration.att.form"));
					attmentIds.add(attachment.getFileUrl());
					attachmentVOs.add(vo);
				}
			}
		}
		// 表单中的附件
		if (formAttrIds != null && formAttrIds.size() > 0) {
			List<CtpContentAll> contentList = GovdocContentHelper.getContentListByModuleId(summaryId);
			if(contentList!=null && contentList.size()>0){
				for (int k = 0; k < contentList.size(); k++) {	
					Integer contentType=contentList.get(k).getContentType();
					if(contentType!=null && contentType.intValue()==20){//当其中一个正文是表单时则过滤表单中的附件
						for (int i = 0; i < formAttrIds.size(); i++) {
							vo = new AttachmentVO();
							try {
								Attachment attachment = this.attachmentManager.getAttachmentByFileURL(Long.valueOf(formAttrIds.get(i).toString()));
								GovdocContentHelper.createAttachmentVO(vo, attachment);
								V3XFile file = fileManager.getV3XFile(Long.valueOf(formAttrIds.get(i).toString()));
								String name = "";
								if (file != null) {
									V3xOrgMember member = orgManager.getMemberById(file.getCreateMember());
									name = member.getName();
								}
								vo.setUserName(name);
								// 表单控件（来自表单控件的属于正文区域）
								vo.setFromType(ResourceUtil.getString("collaboration.att.form"));
								attmentIds.add(attachment.getFileUrl());
								attachmentVOs.add(vo);
							} catch (Exception e) {
								LOGGER.error("获取表单中的附件报错！", e);
							}
						}
					}
				}
			}
		}

		// 添加正文区域中的附件
		if (attmentList != null && attmentList.size() > 0) {
			for (int i = 0; i < attmentList.size(); i++) {
				vo = new AttachmentVO();
				Long fileId = Long.parseLong(attmentList.get(i));
				if (fileId != null) {
					try {
						Attachment attachment = this.attachmentManager.getAttachmentByFileURL(fileId);
						GovdocContentHelper.createAttachmentVO(vo, attachment);
						V3XFile file = fileManager.getV3XFile(fileId);
						V3xOrgMember member = orgManager.getMemberById(file.getCreateMember());
						String name = member.getName();
						vo.setUserName(name);
						// 正文区
						vo.setFromType(ResourceUtil.getString("collaboration.att.form"));
						attmentIds.add(attachment.getFileUrl());
						attachmentVOs.add(vo);
					} catch (Exception e) {
						LOGGER.error("获取正文区域中的附件报错！", e);
					}
				}

			}

		}
		//收藏
        String collectFlag = SystemProperties.getInstance().getProperty("doc.collectFlag");
        if("true".equals(collectFlag) && AppContext.hasPlugin("doc")){
            List<Map<String,Long>> collectMap = docApi.findFavorites(AppContext.currentUserId(),attmentIds);
            Map<Long,Long> doc2SourceId = new HashMap<Long,Long>();
            for (Map<String, Long> map : collectMap) {
                doc2SourceId.put(map.get("sourceId"), map.get("id"));
            }
            for (AttachmentVO attachment : attachmentVOs) {
                if(doc2SourceId.get(Long.valueOf(attachment.getFileUrl()))!=null){
                    attachment.setCollect(true);
                }
            }
        }
		// 排序
		Collections.sort(attachmentVOs, new Comparator<AttachmentVO>(){

			@Override
			public int compare(AttachmentVO o1, AttachmentVO o2) {
				return o1.getUploadTime().compareTo(o2.getUploadTime());
			}
			
		});
		return attachmentVOs;
	}
	
	@SuppressWarnings("rawtypes")
	@Override
	public List<GovdocAttachmentVO> getAttachmentListBySummaryIdForMobile(Long summaryId, String formAttrUrls, String isHistoryFlag) throws BusinessException {
		List<Attachment> tempattachments = null;
		List<Attachment> attachments = new ArrayList<Attachment>();
		List<GovdocAttachmentVO> attachmentVOs = new ArrayList<GovdocAttachmentVO>();
		// 收藏时候传入的文件Id
		List<Long> attmentIds = new ArrayList<Long>();
		GovdocAttachmentVO vo = null;
		HttpServletRequest request = AppContext.getRawRequest();
		List formAttrIdList = null;
		// 获取正文组件中自带的附件
		String attmentContent = request.getParameter("attmentList");
		List<String> attmentList = new ArrayList<String>();
		List<String> _attmentList = new ArrayList<String>();
		if (Strings.isNotBlank(attmentContent) && !"null".equals(attmentContent)) {
			_attmentList = Arrays.asList(attmentContent.split(","));
			for (int a = _attmentList.size() - 1; a > -1; a--) {
				if (!attmentList.contains(_attmentList.get(a))) {
					attmentList.add(_attmentList.get(a));
				}
			}
		}
		if (Strings.isNotBlank(formAttrUrls)) {
			formAttrIdList = Arrays.asList(formAttrUrls.split(","));
		}
		EdocSummary colSummary = govdocSummaryManager.getSummaryById(summaryId);
		tempattachments = attachmentManager.getByReference(summaryId);
		
		//附件列表显示顺序按照sort显示
		if (!tempattachments.isEmpty()) {
			Collections.sort(tempattachments, new Comparator<Object>() {
				@Override
				public int compare(Object o1, Object o2) {
                    Attachment stu1 = (Attachment) o1;
                    Attachment stu2 = (Attachment) o2;
                    int compareResult =  Long.valueOf(stu1.getCreatedate().getTime()).compareTo(Long.valueOf(stu2.getCreatedate().getTime()));

                    if(compareResult == 0)
                    {
                        return Integer.valueOf(stu1.getSort()).compareTo(Integer.valueOf(stu2.getSort()));
                    }
                    return compareResult;
				}
			});
		}
		
		// 控制隐藏的评论对发起人可见
		AppContext.putThreadContext(Comment.THREAD_CTX_NOT_HIDE_TO_ID_KEY, colSummary.getStartMemberId());

		// 添加附件到对象中，附件的type为0，关联文档的type为2（不显示关联文档在附件列表中）
		for (Attachment attachment : tempattachments) {
//			if (attachment.getType() == 0) {
				attachments.add(attachment);
//			}
		}
		if (attachments != null && attachments.size() > 0) {
			List<Comment> comments = govdocCommentManager.getCommentAllByModuleId(ModuleType.edoc, summaryId, "true".equals(isHistoryFlag));
			//List<Comment> childComments = new ArrayList<Comment>();
			for (Attachment attachment : attachments) {
				vo = new GovdocAttachmentVO();
				GovdocContentHelper.createGovdocAttachmentVO(vo, attachment);
				// 创建人
				Long commentId = attachment.getSubReference();
				Comment curComment = null;
				for (Comment comment : comments) {
					if (comment.getId().longValue() == commentId.longValue()) {
						curComment = comment;
						break;
					}
				}
				/**
				 * 1、发起人上传：标题下方的由发起人上传的附件，如果流程中间的节点跟发起人是同一个人，也算”处理人回复“
				 * 2、处理人回复：意见区域的附件（评论&回复），通过修改附件区域上传的附件 3、表单控件：表单正文里面的附件
				 * 4、附言补充：发起人附言区域里面的附件
				 */

				if (curComment != null) {
					if (!curComment.isCanView())
						continue;
					// 来源
					FromType fromType = FromType.title;
					Integer cType = curComment.getCtype();
					// 如果是草稿状态时，不显示附件
					if (cType != null) {
						int ct = cType.intValue();
						if (ct == -2) {
							continue;
						}
						if (curComment.getCtype().intValue() == Comment.CommentType.govdocniban.getKey())// 如果是拟办意见的，不显示
							continue;
						// 如果是转发的则显示成正文区
						if (curComment.getForwardCount() > 0) {
							fromType = FromType.form;
						} else {
							if (ct == -1) {
								fromType = FromType.sender;
							} else if (ct == 1 || ct == 0) {
								fromType = FromType.reply;// "处理区";
							}
						}
					}
					attmentIds.add(attachment.getFileUrl());
					vo.setFrom(fromType.ordinal());
					attachmentVOs.add(vo);
				} else if (summaryId.equals(attachment.getSubReference())) {
					// 标题区
					Date attaDate = attachment.getCreatedate();
					Date colDate = colSummary.getCreateTime();
					// 将协同创建时间加上2秒,以判断是否是发起人上传，因为在发起人上传时，必须保证附件的创建时间小于协同的创建时间
					// 但是在实际情况中由于保存时都是用的new Date()方式存储日期，导致时间有误差(ms级)
					FromType fromType = FromType.title;
					if (attaDate.getTime() < colDate.getTime() + 2000) {
						fromType = FromType.title;
					}
					vo.setFrom(fromType.ordinal());
					attmentIds.add(attachment.getFileUrl());
					attachmentVOs.add(vo);
				} else if (AttachmentEditUtil.CONTENT_ATTACHMENTSUBRE.equals(attachment.getSubReference().toString())) { // 来之转发表单控件的附件
					// （来自表单转发控件的属于正文区域）
					vo.setFrom(FromType.form.ordinal());
					attmentIds.add(attachment.getFileUrl());
					attachmentVOs.add(vo);
				}
			}
		}
		// 表单中的附件
		if (formAttrIdList != null && formAttrIdList.size() > 0) {
			List<CtpContentAll> contentList = GovdocContentHelper.getContentListByModuleId(summaryId);
			if(contentList!=null && contentList.size()>0){
				for (int k = 0; k < contentList.size(); k++) {	
					Integer contentType=contentList.get(k).getContentType();
					if(contentType != null && contentType.intValue() == 20){//当其中一个正文是表单时则过滤表单中的附件
						for (int i = 0; i < formAttrIdList.size(); i++) {
							vo = new GovdocAttachmentVO();
							try {
								Attachment attachment = this.attachmentManager.getAttachmentByFileURL(Long.valueOf(formAttrIdList.get(i).toString()));
								vo = new GovdocAttachmentVO();
								GovdocContentHelper.createGovdocAttachmentVO(vo, attachment);
								// 表单控件（来自表单控件的属于正文区域）
								vo.setFrom(FromType.form.ordinal());
								attmentIds.add(attachment.getFileUrl());
								attachmentVOs.add(vo);
							} catch (Exception e) {
								LOGGER.error("获取表单中的附件报错！", e);
							}
						}
					}
				}
			}
		}		

		// 添加正文区域中的附件
		if (attmentList != null && attmentList.size() > 0) {
			for (int i = 0; i < attmentList.size(); i++) {
				vo = new GovdocAttachmentVO();
				Long fileId = Long.parseLong(attmentList.get(i));
				if (fileId != null) {
					try {
						Attachment attachment = this.attachmentManager.getAttachmentByFileURL(fileId);
						GovdocContentHelper.createGovdocAttachmentVO(vo, attachment);
						// 正文区
						vo.setFrom(FromType.form.ordinal());
						attmentIds.add(attachment.getFileUrl());
						attachmentVOs.add(vo);
					} catch (Exception e) {
						LOGGER.error("获取正文区域中的附件报错！", e);
					}
				}

			}
		}
		return attachmentVOs;
	}
	
	@Override
	public String getSummaryAttachmentJsonsIncludeSender(long summaryId,GovdocNewVO info) throws BusinessException {
		// 取正文的附件和发起人附言的。
		List<Attachment> showAtts = new ArrayList<Attachment>();
		//表单上的附件
		List<Attachment> formAtt = new ArrayList<Attachment>();
		//正文区除去关联文档的附件
		List<Attachment> contentAtt = new ArrayList<Attachment>();
		List<Attachment> list = attachmentManager.getByReference(summaryId);
		List<Comment> comments = govdocCommentManager.getCommentList(ModuleType.edoc, summaryId);
		List<Long> showlds = new ArrayList<Long>();
		showlds.add(summaryId);
		if (Strings.isNotEmpty(comments)) {
			for (Comment c : comments) {
				if (Integer.valueOf(Comment.CommentType.sender.getKey()).equals(c.getCtype())) {
					showlds.add(c.getId());
				}
			}
		}
		if (Strings.isNotEmpty(list)) {
			for (Attachment a : list) {
				if (showlds.contains(a.getSubReference())) {
					Attachment aclone = null;
					try {
						aclone = (Attachment) a.clone();
						aclone.setSubReference(a.getReference());
					} catch (CloneNotSupportedException e) {
						LOGGER.error("", e);
					}
					if (aclone != null) {
						showAtts.add(aclone);
						if(aclone.getType() != 2){
							contentAtt.add(aclone);
						}
					}
				}else{
					if(a.getCategory() == ApplicationCategoryEnum.form.getKey()){
						formAtt.add(a);
					}
				}
			}
		}
		if(info != null){
			//表单中附件的文件名(附件说明用到)
			String formAttsName = attachmentManager.getFileNameExcludeSuffix(formAtt);
			info.setFilesmFormAttsName(formAttsName);
			//正文区附件的文件名(附件说明用到)
			String contentAttsName = attachmentManager.getFileNameExcludeSuffix(contentAtt);
			info.setFilesmContentAttsName(contentAttsName);
			
		}
		
		String attListJSON = attachmentManager.getAttListJSON(showAtts);
		return attListJSON;
	}
	/*************************** 33333 公文附件查看 start ***************************/
	
	
	/*************************** 44444 公文附件保存 start ***************************/
	@Override
	public void saveAttachments(GovdocNewVO info, SendType sendType) throws BusinessException {
		// 保存上传的附件
		boolean attaFlag = false;
		EdocSummary summary = info.getSummary();
		if (!EdocConstant.SendType.auto.equals(sendType) && !EdocConstant.SendType.child.equals(sendType)) {
			try {
				String attaFlagStr = "";
				ApplicationCategoryEnum appenum = ApplicationCategoryEnum.edoc;
				if (sendType == EdocConstant.SendType.forward) {
					HttpServletRequest request = (HttpServletRequest) AppContext.getThreadContext(GlobalNames.THREAD_CONTEXT_REQUEST_KEY);
					request.removeAttribute("HASSAVEDATTACHMENT");// 保存附件之前清空这个标记
					attaFlagStr = this.attachmentManager.create(appenum, summary.getId(), summary.getId());
				} else if (sendType == EdocConstant.SendType.resend || sendType == EdocConstant.SendType.normal) {
					if (info.getNibanComment() != null) {// 如果有拟办意见，则将附件也保存到拟办意见中
						attaFlagStr = saveAttachmentFromDomain(appenum, info);
						if (Strings.isNotBlank(info.getNibanComment().getRelateInfo())) {// 先将该值置为空，否则保存两次附件会出错
							info.getNibanComment().setRelateInfo("");
						}
					} else {
						attaFlagStr = saveAttachmentFromDomain(appenum, info);
					}

				}
				attaFlag = Constants.isUploadLocaleFile(attaFlagStr);
				if (attaFlag) {
					summary.setHasAttachments(attaFlag);
				}
			} catch (Exception e) {
				LOGGER.info("保存上传的附件报错!", e);
			}
		}
	}
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private String saveAttachmentFromDomain(ApplicationCategoryEnum type, GovdocNewVO info) throws BusinessException {
		List assDocGroup = ParamUtil.getJsonDomainGroup("assDocDomain");
		int assDocSize = assDocGroup.size();
		Map assDocMap = ParamUtil.getJsonDomain("assDocDomain");
		if (assDocSize == 0 && assDocMap.size() > 0) {
			assDocGroup.add(assDocMap);
		}

		List attFileGroup = ParamUtil.getJsonDomainGroup("attFileDomain");
		int attFileSize = attFileGroup.size();
		Map attFileMap = ParamUtil.getJsonDomain("attFileDomain");
		if (attFileSize == 0 && attFileMap.size() > 0) {
			attFileGroup.add(attFileMap);
		}

		assDocGroup.addAll(attFileGroup);

		List result;
		try {
			Long moduleId = info.getSummary().getId();
			result = attachmentManager.getAttachmentsFromAttachList(type, moduleId, moduleId, assDocGroup);
			if (org.apache.commons.collections.CollectionUtils.isNotEmpty(result)) {
				List<Attachment> newAtts = new ArrayList<Attachment>();
				for (Attachment att : (List<Attachment>) result) {
					Attachment newAtt = (Attachment) att.clone();
					newAtt.setId(UUIDLong.longUUID());
					newAtt.setReference(moduleId);
					newAtt.setFilename(att.getFilename());
					newAtt.setFilename(newAtt.getFilename().replace(' ', ' ').replace("\'", "\\\'"));
					newAtt.setCategory(ApplicationCategoryEnum.edoc.getKey());
					if (info.getNibanComment() != null) {
						newAtt.setSubReference(info.getNibanComment().getId());
					}
					newAtts.add(newAtt);
				}
				if (info.getNibanComment() != null) {
					info.getNibanComment().setRelateInfo(JSONUtil.toJSONString(attachmentManager.setOfficeTransformEnable(newAtts)));
				}
			}
		} catch (Exception e) {
			LOGGER.error(e.getMessage(), e);
			throw new BusinessException("创建附件出错");
		}

		return attachmentManager.create(result);
	}
	
    /**
     * 修改附件的保存
     * @param summary
     * @param attaFlag
     * @param affair
     * @param toSendMsg 是否发送消息
     * @return
     * @throws BusinessException
     */
    @SuppressWarnings({ "rawtypes", "unchecked" })
	private String saveAttachment(EdocSummary summary, CtpAffair affair, boolean toSendMsg,Long commentId) throws BusinessException {
    	 String attaFlag = "";
         try {
             //保存附件之前先删除原来到附件

         	List attFileGroup = ParamUtil.getJsonDomainGroup("attFileDomain");
             int attFileSize = attFileGroup.size();
             Map attFileMap = ParamUtil.getJsonDomain("attFileDomain");
             if (attFileSize == 0 && attFileMap.size() > 0) {
             	attFileGroup.add(attFileMap);
             }
             List assDocGroup = ParamUtil.getJsonDomainGroup("assDocDomain");
             int assDocSize = assDocGroup.size();
             Map assDocMap = ParamUtil.getJsonDomain("assDocDomain");
             if (assDocSize == 0 && assDocMap.size() > 0) {
             	assDocGroup.add(assDocMap);
             }
             attFileGroup.addAll(assDocGroup);
             List<Attachment> newAtts = attachmentManager.getAttachmentsFromAttachList(ApplicationCategoryEnum.collaboration, summary.getId(), summary.getId(), attFileGroup);
             if(Strings.isNotEmpty(attFileGroup)) {
             	for(int i = 0 ; i<attFileGroup.size() ; i++ ) {
             		  Object o = attFileGroup.get(i);
             		  if (o instanceof Map) {
             			  Map map = (Map) o;
             			  String id = (String) map.get("attachment_id");
             			  if(Strings.isNotBlank(id)) {
             				  newAtts.get(i).setId(Long.valueOf(id));
             			  }
             		  }
             	}
             }
             
             
             List<Attachment> dbAtts = attachmentManager.getByReference(summary.getId(), summary.getId());
             
             List<Attachment> needAddAtts = new ArrayList<Attachment>();
             
            // List<Long> newAttIds = new ArrayList<Long>();
             Map<Long,Attachment> newAttMap = new HashMap<Long,Attachment>();
             
             Map<Long,Attachment> dbAttMap = new HashMap<Long,Attachment>();
             ///Set<Long> oldAttFUIds = new HashSet<Long>();

             LOGGER.info("===============当前附件：");
             for(Attachment newAtt :newAtts ){
 				/*
 				 * if(!newAttIds.contains(newAtt.getFileUrl())){
 				 * newAttIds.add(newAtt.getFileUrl()); }
 				 */
                 newAttMap.put(newAtt.getId(), newAtt);
                 LOGGER.info("文件名：" + newAtt.getFilename()+",ID："+newAtt.getId()+"创建时间："+newAtt.getCreatedate());
             }

             LOGGER.info("原来的附件：");
             for(Attachment oldAtt :dbAtts ){
                 //oldAttFUIds.add(oldAtt.getFileUrl());
                 dbAttMap.put(oldAtt.getId(),oldAtt);
                 LOGGER.info("文件名：" + oldAtt.getFilename()+",ID："+oldAtt.getId()+"创建时间："+oldAtt.getCreatedate());
             }

             
             for(Long id : newAttMap.keySet()){
                 if(!dbAttMap.containsKey(id)) {
                 	if(!needAddAtts.contains(newAttMap.get(id))){
                 		needAddAtts.add(newAttMap.get(id));
                 	}
                 	LOGGER.info("添加的附件：" + id);
                 }
               //原来的附件，比较文件名
 				if(dbAttMap.containsKey(id)){
 					Attachment oldAtt = dbAttMap.get(id);
 					Attachment newAtt = newAttMap.get(id);
 					if(!oldAtt.getFilename().equals(newAtt.getFilename())){
 						V3XFile v3XFile = fileManager.getV3XFile(oldAtt.getFileUrl());
 						oldAtt.setFilename(newAtt.getFilename());
						oldAtt.setCreatedate(new Date());
						if( null != v3XFile){
							v3XFile.setFilename(newAtt.getFilename());
							v3XFile.setUpdateDate(oldAtt.getCreatedate());
							fileManager.update(v3XFile);
						}
 						attachmentManager.update(oldAtt);
 						LOGGER.info("重命名的附件fileUrl：" + id);
 					}
 				}
             }

             for(Long id : dbAttMap.keySet()){
                 if(!newAttMap.containsKey(id)){
                     attachmentManager.deleteById(id);
                     LOGGER.info("删除附件 ："+id);
                 }
             }

             //修改附件的文件确保修改前后文件Id是一样的。
             for(Attachment newAtt : newAttMap.values()) {
             	
             	if(newAtt.getType() != com.seeyon.ctp.common.filemanager.Constants.ATTACHMENT_TYPE.FILE.ordinal()) {
             		continue;
             	}
             	
             	Long newAttId = newAtt.getId();
             	Attachment thisOldAtt = dbAttMap.get(newAttId);
             	
             	//如果FileURL不相同，则证明修改了这个文件，保证附件的文件ID和以前的是一样的；
             	if(thisOldAtt != null  && !newAtt.getFileUrl().equals(thisOldAtt.getFileUrl())) {
             		
             		//更新V3x属性
             		V3XFile v3XFile = fileManager.getV3XFile(thisOldAtt.getFileUrl());
             		
             		//备份源文件
             		String filePath = this.fileManager.getFolder(v3XFile.getUpdateDate(), true)+ File.separator + thisOldAtt.getFileUrl();
             		String destPath = this.fileManager.getFolder(newAtt.getCreatedate(), true)+ File.separator + thisOldAtt.getFileUrl();
             		
             		bakPhysicalFile(destPath,filePath);
             		
             		//将新的文件拷贝为原来的文件;
             		fileManager.clone(newAtt.getFileUrl(), newAtt.getCreatedate(), thisOldAtt.getFileUrl(), newAtt.getCreatedate());
             		
             		//删除原来的附件
             		fileManager.deletePhysicsFile(newAtt.getFileUrl());
             		
             		//清除office转化，让下次点击的时候是新的；
             		officeTransManager.clean(thisOldAtt.getFileUrl(), Datetimes.format(thisOldAtt.getCreatedate(), "yyyyMMdd")); 
             	
             		v3XFile.setUpdateDate(newAtt.getCreatedate());
             		newAtt.setCreatedate(thisOldAtt.getCreatedate());//设置为老的创建时间
             		v3XFile.setCreateDate(thisOldAtt.getCreatedate());
             		v3XFile.setCreateMember(AppContext.currentUserId());
             		fileManager.update(v3XFile);
             		
             	}
             }
             
             
             attaFlag = attachmentManager.create(needAddAtts,false);
             
             LOGGER.info("添加附件成功返回的attaFlag:" + attaFlag);

         	AttachmentEditUtil attUtil = new AttachmentEditUtil("attActionLogDomain");
         	List<ProcessLog> logs = attUtil.parseProcessLog(Long.valueOf(summary.getProcessId()), affair.getActivityId());
         	for(ProcessLog log : logs){
         		log.setCommentId(commentId);
         		//处理附件编辑重命名后出现特殊字符%C2%A0(空格变成了问号)
         		String contentString = log.getParam0();
         		contentString = contentString.replace(" "," ");
             	log.setParam0(contentString);
             	//处理附件编辑重命名后出现特殊字符%C2%A0(空格变成了问号) end 
         	}
         	
             govdocLogManager.insertProcessLog(logs);

         	updateSummaryAttachment(newAtts.size(),summary,affair, toSendMsg);

         	//修改附件更新全文检索库
             if(AppContext.hasPlugin("index")) {
                 indexApi.update(affair.getObjectId(), ApplicationCategoryEnum.edoc.getKey());
             }
         } catch (Exception e) {
         	LOGGER.error("创建附件出错，位于方法ColManagerImpl.saveAttachment", e);
         	throw new BusinessException("创建附件出错");
         }
         return attaFlag;
    }
    private void updateSummaryAttachment(int attSize, EdocSummary summary, CtpAffair affair, boolean toSendMsg) throws BusinessException{
    	//Map<String, String> colSummaryDomian = (Map<String, String>)ParamUtil.getJsonDomain("colSummaryData");
    	//int type=0;
    	boolean needUpdate = false;
    	boolean isHasAtt=summary.isHasAttachments();
    	if(!isHasAtt && attSize > 0){
    		needUpdate=true;
    		//更新summary的附件标志位
    		GovdocUtil.setHasAttachments(summary, true);
    	}else if(isHasAtt && attSize == 0){
    		//更新summary的附件标志位
    		needUpdate=true;
    		GovdocUtil.setHasAttachments(summary, false);
    	}
    	if(needUpdate){
    		//更新该流程中所有的待办标志位
			Map<String,Object> parameter = new HashMap<String,Object>();
			parameter.put("identifier", summary.getIdentifier());
			affair.setIdentifier(summary.getIdentifier());
			govdocSummaryManager.updateEdocSummary(summary);
			affairManager.updateAffairs(ApplicationCategoryEnum.edoc, summary.getId(), parameter);
		}
    	if(toSendMsg){
    	    govdocMessageManager.sendMessage4ModifyBodyOrAtt(summary, affair, 0);
    	}
	}
    
    @Override
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public String saveAttachment(EdocSummary summary, CtpAffair affair ,Long commentId) throws BusinessException {
		String attaFlag = "";
		try {
			// 保存附件之前先删除原来到附件

			List attFileGroup = ParamUtil.getJsonDomainGroup("attFileDomain");
			List<Object> removeList = new ArrayList<Object>();
			for(Object att :attFileGroup){
				//chenyq 一屏式正式附件里面有表单附件
				if(att instanceof Map){
				   Map map = (Map)att;
				   if("2".equals(map.get("attachment_category"))){
					   removeList.add(att);
					   //attFileGroup.remove(att);
				   }
				}
			}
			for(Object att :removeList){
				attFileGroup.remove(att);
			}
			int attFileSize = attFileGroup.size();
			Map attFileMap = ParamUtil.getJsonDomain("attFileDomain");
			if (attFileSize == 0 && attFileMap.size() > 0) {
				attFileGroup.add(attFileMap);
			}
			List assDocGroup = ParamUtil.getJsonDomainGroup("assDocDomain");
			int assDocSize = assDocGroup.size();
			Map assDocMap = ParamUtil.getJsonDomain("assDocDomain");
			if (assDocSize == 0 && assDocMap.size() > 0) {
				assDocGroup.add(assDocMap);
			}
			attFileGroup.addAll(assDocGroup);
			List<Attachment> result = attachmentManager.getAttachmentsFromAttachList(ApplicationCategoryEnum.edoc, summary.getId(), summary.getId(),
					attFileGroup);
			// Object obj = summary.getExtraAttr("opinionId");
			// if (null != obj) {
			//
			// result =
			// attachmentManager.getAttachmentsFromRequest(ApplicationCategoryEnum.edoc,
			// summary.getId(),
			// Long.parseLong(obj.toString()), AppContext.getRawRequest());
			// }
			List<Attachment> oldAtts = attachmentManager.getByReference(summary.getId(), summary.getId());
			Set<Attachment> needAddAtts = new HashSet<Attachment>();
			Set<Long> newAttFUIds = new HashSet<Long>();
			Map<Long,Attachment> oldAttsMap = new HashMap<Long,Attachment>();
			Map<Long, Attachment> nm = new HashMap<Long, Attachment>();
			Map<Long, Long> oldFUID = new HashMap<Long, Long>();
			Set<Long> oldAttFUIds = new HashSet<Long>();

			LOGGER.info("===============当前附件：");
			for (Attachment newAtt : result) {
				newAttFUIds.add(newAtt.getFileUrl());
				nm.put(newAtt.getFileUrl(), newAtt);
				LOGGER.info("文件名：" + newAtt.getFilename() + ",ID：" + newAtt.getId() + "创建时间：" + newAtt.getCreatedate());
			}

			LOGGER.info("原来的附件：");
			for (Attachment oldAtt : oldAtts) {
				oldAttFUIds.add(oldAtt.getFileUrl());
				oldFUID.put(oldAtt.getFileUrl(), oldAtt.getId());
				oldAttsMap.put(oldAtt.getFileUrl(),oldAtt);
				LOGGER.info("文件名：" + oldAtt.getFilename() + ",ID：" + oldAtt.getId() + "创建时间：" + oldAtt.getCreatedate());
			}

			for (Long id : newAttFUIds) {
				if (!oldAttFUIds.contains(id)) {
					needAddAtts.add(nm.get(id));
					LOGGER.info("添加的附件fileUrl：" + nm.get(id));
				}
				//如果原来附件包含新的附件id，分别对比文件名是否修改
				if(oldAttFUIds.contains(id)){
					Attachment oldAtt =  oldAttsMap.get(id);
					Attachment newAtt = nm.get(id);
					if(!oldAtt.getFilename().equals(newAtt.getFilename())){
						V3XFile v3XFile = fileManager.getV3XFile(oldAtt.getFileUrl());
						oldAtt.setFilename(newAtt.getFilename());
						oldAtt.setCreatedate(new Date());
						if( null != v3XFile){
							v3XFile.setFilename(newAtt.getFilename());
							v3XFile.setUpdateDate(oldAtt.getCreatedate());
							fileManager.update(v3XFile);
						}
						attachmentManager.update(oldAtt);
						LOGGER.info("重命名附件fileUrl：" + id);
					}
				}
			}

			for (Long id : oldAttFUIds) {
				if (!newAttFUIds.contains(id)) {
					attachmentManager.deleteById(oldFUID.get(id));
					LOGGER.info("删除附件 ：" + oldFUID.get(id));
				}
			}

			attaFlag = attachmentManager.create(needAddAtts,false);
			LOGGER.info("添加附件成功返回的attaFlag:" + attaFlag);

			AttachmentEditUtil attUtil = new AttachmentEditUtil("attActionLogDomain");

			if(commentId != null){
	        	List<ProcessLog> logs = attUtil.parseProcessLog(Long.valueOf(summary.getProcessId()), affair.getActivityId());
	        	for(ProcessLog log : logs){
	        		log.setCommentId(commentId);
	        	}
	            govdocLogManager.insertProcessLog(logs);
			}

			updateSummaryAttachment(result.size(), summary, affair);
			// 修改附件更新全文检索库
			if (AppContext.hasPlugin("index")) {
			    indexApi.update(affair.getObjectId(), ApplicationCategoryEnum.edoc.getKey());
			}
		} catch (Exception e) {
			LOGGER.error("创建附件出错，位于方法ColManagerImpl.saveAttachment", e);
			throw new BusinessException("创建附件出错");
		}
		return attaFlag;
	}
	
    private void bakPhysicalFile(String destPath,String originalPath) {
		// 公文正文备份
		//命名规则：原文件名_时刻（到秒）.bak
		//存放路径：Office正文原始文件路径下
		//比如原始office正文存放在e:\\ufseeyon\group\base\\upload\2010\01\20下，则备份文件也放到这个路径下，主要方便运维同事查找，存在的问题是不能做增量备份
		//todo:流程结束，备份文件删除
		try {
			String now = Datetimes.format(new Date(), "yyyyMMddHHmmss");
			String contentFileBak=destPath+"_"+now+".bak";
			File f=new File(originalPath);
			if(f.exists()){
				
				FileUtils.copyFile(f, new File(contentFileBak));
			}
		}
		catch (Exception e) {
			LOGGER.error("公文正文内容备份异常 ：" , e);
		}
	}
    
	private void updateSummaryAttachment(int attSize, EdocSummary summary, CtpAffair affair) throws BusinessException {
		boolean needUpdate = false;
		boolean isHasAtt = summary.isHasAttachments();
		if (!isHasAtt && attSize > 0) {
			needUpdate = true;
			// 更新summary的附件标志位
			summary.setHasAttachments(true);
			AffairUtil.setHasAttachments(affair, true);
		} else if (isHasAtt && attSize == 0) {
			// 更新summary的附件标志位
			needUpdate = true;
			summary.setHasAttachments(false);
			AffairUtil.setHasAttachments(affair, false);
		}
		if (needUpdate) {
			// 更新该流程中所有的待办标志位
			Map<String, Object> parameter = new HashMap<String, Object>();
			parameter.put("identifier", summary.getIdentifier());
			govdocSummaryManager.updateEdocSummary(summary,false);
			affairManager.updateAffairs(ApplicationCategoryEnum.edoc, summary.getId(), parameter);
		}

		// 发送消息
		// colMessageManager.saveUpdataAttMessage(affairManager,
		// userMessageManager, orgManager, summary, affair);
	}
	
	@Override
	public void saveEdocOpinionRelationAttachment(int i, long reference, long subReference, int type, ApplicationCategoryEnum cotegory, String fileName, String mimeType, long fileUrl) throws Exception {
		Attachment e = new Attachment();
		e.setIdIfNew();
		e.setSort(i);
		e.setReference(reference);
		e.setSubReference(subReference);
		e.setCategory(Integer.valueOf(cotegory.getKey()));
		e.setType(type);
		e.setFilename(fileName);
		e.setMimeType(mimeType);
		e.setFileUrl(fileUrl);
		e.setCreatedate(new Date());
		e.setSize(0L);
		e.setDescription(String.valueOf(fileUrl));
		this.attachmentDAO.save(e);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	public String saveAttachmentFromDomain(ApplicationCategoryEnum type, Long module_id) throws BusinessException {
		List assDocGroup = ParamUtil.getJsonDomainGroup("assDocDomain");
		int assDocSize = assDocGroup.size();
		Map assDocMap = ParamUtil.getJsonDomain("assDocDomain");
		if (assDocSize == 0 && assDocMap.size() > 0) {
			assDocGroup.add(assDocMap);
		}

		List attFileGroup = ParamUtil.getJsonDomainGroup("attFileDomain");
		int attFileSize = attFileGroup.size();
		Map attFileMap = ParamUtil.getJsonDomain("attFileDomain");
		if (attFileSize == 0 && attFileMap.size() > 0) {
			attFileGroup.add(attFileMap);
		}

		assDocGroup.addAll(attFileGroup);

		List result;
		try {
			result = attachmentManager.getAttachmentsFromAttachList(ApplicationCategoryEnum.edoc, module_id, module_id, assDocGroup);
		} catch (Exception e) {
			LOGGER.error("", e);
			throw new BusinessException("创建附件出错");
		}

		return attachmentManager.create(result);
	}

	@SuppressWarnings("unchecked")
	@Override
	public void saveAttDatas(GovdocBaseVO baseVo) throws BusinessException{
	      //保存附件和正文修改
        Map<String, String> colSummaryDomian = (Map<String, String>)ParamUtil.getJsonDomain("colSummaryData");
        if(colSummaryDomian != null) {
            AttachmentEditUtil attUtil = new AttachmentEditUtil("attActionLogDomain");
            boolean modifyContent = "1".equals(colSummaryDomian.get("modifyFlag"));
            boolean modifyAtt = attUtil.hasEditAtt();

            EdocSummary summary = baseVo.getSummary();
            Long commentId = baseVo.getComment().getId();
            CtpAffair affair = baseVo.getAffair();
            User user = baseVo.getCurrentUser();
            if(modifyAtt) {
				saveAttachment(summary, baseVo.getAffair(), !modifyContent, commentId);
            }
            if(modifyContent){
                //修改正文后记录流程日志
                processLogManager.insertLog(AppContext.getCurrentUser(), Long.parseLong(summary.getProcessId()),
                        affair.getActivityId(), ProcessLogAction.processEdoc,commentId,String.valueOf(ProcessLogAction.ProcessEdocAction.modifyBody.getKey()));
                
                //修改正文发送消息
                if(!modifyAtt){
                    govdocMessageManager.sendMessage4ModifyBodyOrAtt(summary, affair, 1);
                }
                //如果修改正文的时候导入了新文件则记录应用日志
                if("1".equals(colSummaryDomian.get("isLoadNewFile"))){
                    appLogManager.insertLog(user, AppLogAction.Coll_Content_Edit_LoadNewFile, user.getName(), affair.getSubject());
                }
            }
            
            //同时修改了正文和附件发送消息
            if(modifyAtt && modifyContent){
            	govdocMessageManager.sendMessage4ModifyBodyOrAtt(summary, affair, 2);
            }
        }
    }
	@Override
    public void saveAttDatas(User user, EdocSummary summary, CtpAffair affair, Long commentId) throws BusinessException{
      //保存附件和正文修改
        @SuppressWarnings("unchecked")
		Map<String, String> colSummaryDomian = (Map<String, String>)ParamUtil.getJsonDomain("colSummaryData");
        if(colSummaryDomian != null){

            AttachmentEditUtil attUtil = new AttachmentEditUtil("attActionLogDomain");
            boolean modifyContent = "1".equals(colSummaryDomian.get("modifyFlag"));
            boolean modifyAtt = attUtil.hasEditAtt();

            if(modifyAtt){
                this.saveAttachment(summary, affair,commentId);
            }

            if(modifyContent){
                //修改正文后记录流程日志
                govdocLogManager.insertProcessLog(AppContext.getCurrentUser(), Long.parseLong(summary.getProcessId()),
                        affair.getActivityId(), ProcessLogAction.processColl,commentId,String.valueOf(ProcessLogAction.ProcessEdocAction.modifyBody.getKey()));

                //修改正文发送消息
                if(!modifyAtt){
                    govdocMessageManager.sendMessage4ModifyBodyOrAtt(summary, affair, 1);
                }
                //如果修改正文的时候导入了新文件则记录应用日志
                if("1".equals(colSummaryDomian.get("isLoadNewFile"))){
                    govdocLogManager.insertAppLog(user, GovdocAppLogAction.COLL_CONTENT_EDIT_LOADNEWFILE.key(), user.getName(), affair.getSubject());
                }
            }

            //同时修改了正文和附件发送消息
            if(modifyAtt && modifyContent){
            	govdocMessageManager.sendMessage4ModifyBodyOrAtt(summary, affair, 2);
            }
        }
    }
	/*************************** 44444 公文附件保存   end ***************************/
	
    
    /*************************** 55555 公文附件删除 start ***************************/
    @Override
    public void deleteAttachment4SpecialBack(Long id) {
		try {
			this.attachmentManager.deleteByReference(id, id);
			List<Long> commentIds = govdocCommentManager.getSenderCommentIdByModuleIdAndCtype(ModuleType.edoc, id);
			if (Strings.isNotEmpty(commentIds)) {
				for (Long cid : commentIds) {
					attachmentManager.deleteByReference(id, cid);
				}
			}
		} catch (BusinessException e) {
			LOGGER.error("指定回退删除附件报错" + e.getLocalizedMessage(), e);
		}
	}
    /*************************** 55555 公文附件删除   end ***************************/

    
    /*************************** 666666 验证正文存在 start ***************************/
    @Override
    public boolean checkBodyExist(GovdocBodyVO bodyVo) throws BusinessException {
    	// 陈应强 判断流程发起时，正文是否存在 start
		if (bodyVo.getContent() != null) {
			if (null != bodyVo.getContentType() && 10 != bodyVo.getContentType()) {// html正文
				// 没有文件
				V3XFile file = fileManager.getV3XFile(Long.parseLong(bodyVo.getContent()));
				if (file != null) {
					if(file.getUpdateDate() != null){
						String filePath = this.fileManager.getFolder(file.getUpdateDate(), true) + File.separator;
						File ftemp = new File(filePath + file.getId());
						if (ftemp.exists() && ftemp.isFile()) {
							return true;
						} else {
							LOGGER.info("正文文件不存在，V3Xfileid为:" + file.getId());
							return false;
						}
					}
					if (Strings.isNotBlank(bodyVo.getFileName())) {
						file.setFilename(bodyVo.getFileName());
						fileManager.save(file);
					}
				} else {
					LOGGER.info("没有获取到文件信息");
				}
			}
		}
		return true;
    }

    @Override
    public String createOFDBodies(String summaryId, String ofdFileId ) throws BusinessException {
		//获取已转换过的ofd正文
		Long summaryIdLong=Long.valueOf(summaryId);
  		MainbodyManager contentManager = MainbodyService.getInstance().getContentManager();
		List<CtpContentAll> contents = contentManager.getContentListByModuleIdAndModuleType(ModuleType.edoc,summaryIdLong);
  		if(null==contents||contents.isEmpty()){
  			return "failed";
  		}
  		CtpContentAll content=null;
  		for(CtpContentAll c:contents){
  			if(c.getContentType() == MainbodyType.Ofd.getKey()){
  				content=c;
  				break;
  			}
  		}
  		//第一次转则新增ofd正文
  		if (null==content){
  			content=new CtpContentAll();
  			content.setNewId();
  		}
  		content.setModuleId(summaryIdLong);
  		content.setCreateId(AppContext.getCurrentUser().getId());
  		content.setContentTemplateId(0L);
  		content.setCreateDate(new java.util.Date());
  		content.setContent(ofdFileId);
  		content.setSort(3);
  		content.setContentType(MainbodyType.Ofd.getKey());
  		content.setModuleType(ModuleType.edoc.getKey()); // 公文类型
  	    try {
			contentManager.saveOrUpdateContentAll(content);
		} catch (Exception e) {
			LOGGER.error("OFD保存到CtpContentAll出错！", e);
			return "failed";
		}
  	    return "success";
	}
    
    /********************************* 查询花脸    end **********************************/
    @Override
	public List<V3XFile> findBackFileIds(String fileId){
		List<V3XFile> fs = new ArrayList<V3XFile>();
		List<Long> obfList = new ArrayList<Long>();
		if(fileId.indexOf(".") != -1){
			fileId = fileId.substring(0,fileId.indexOf("."));
		}
		if (NumberUtils.isNumber(fileId)) {//fn是文件Id,不是文件名时
			obfList = officeBakFileManager.getOfficeBakFileIds(Long.parseLong(fileId));
		}
		try {
			Long[] fileIds = null;
			if (obfList.size() > 0) {
				fileIds = new Long[obfList.size()];
				for (int i = 0; i < obfList.size(); i++) {
					fileIds[i] = obfList.get(i);
				}
				fs = fileManager.getV3XFile(fileIds);
			}
			//花脸根据时间进行倒序显示
			Collections.sort(fs,  new Comparator<V3XFile>(){
				@Override
				public int compare(V3XFile o1, V3XFile o2) {
					if(o1.getCreateDate().before(o2.getCreateDate())){
						return 1;
					}
					return -1;
				}
			});
			
		} catch (BusinessException e) {
			LOGGER.error("findBackFileIds报错", e);
		}
		return fs;
	}
	/********************************* 查询花脸    end **********************************/
    
    /*************************** 666666 验证正文存在    end ***************************/
      
	/*************************** 99999 Spring注入，请将业务写在上面 start ******************************/
	public void setCtpMainbodyManager(MainbodyManager ctpMainbodyManager) { this.ctpMainbodyManager = ctpMainbodyManager; }
	public void setAffairManager(AffairManager affairManager) { this.affairManager = affairManager; }
	public void setGovdocSummaryManager(GovdocSummaryManager govdocSummaryManager) { this.govdocSummaryManager = govdocSummaryManager; }
	public void setGovdocSignetManager(GovdocSignetManager govdocSignetManager) { this.govdocSignetManager = govdocSignetManager; }
	public void setGovdocLogManager(GovdocLogManager govdocLogManager) { this.govdocLogManager = govdocLogManager; }
	public void setIndexApi(IndexApi indexApi) { this.indexApi = indexApi; }
	public void setFileManager(FileManager fileManager) { this.fileManager = fileManager; }
	public void setGovdocCommentManager(GovdocCommentManager govdocCommentManager) { this.govdocCommentManager = govdocCommentManager; }
	public void setAttachmentManager(AttachmentManager attachmentManager) { this.attachmentManager = attachmentManager; }
	public void setAttachmentDAO(AttachmentDAO attachmentDAO) { this.attachmentDAO = attachmentDAO; }
	public void setOrgManager(OrgManager orgManager) { this.orgManager = orgManager; }
	public void setDocApi(DocApi docApi) { this.docApi = docApi; }
	public void setGovdocMessageManager(GovdocMessageManager govdocMessageManager) { this.govdocMessageManager = govdocMessageManager; }
	public void setWapi(WorkflowApiManager wapi) {this.wapi = wapi;}
	public void setOfficeBakFileManager(OfficeBakFileManager officeBakFileManager) {this.officeBakFileManager = officeBakFileManager;}
	public void setProcessLogManager(ProcessLogManager processLogManager) {
        this.processLogManager = processLogManager;
    }

	public void setAppLogManager(AppLogManager appLogManager) {
		this.appLogManager = appLogManager;
	}
	
	/*************************** 99999 Spring注入，请将业务写在上面   end ******************************/	

	public OfficeTransManager getOfficeTransManager() {
		return officeTransManager;
	}

	public void setOfficeTransManager(OfficeTransManager officeTransManager) {
		this.officeTransManager = officeTransManager;
	}
}
