package com.seeyon.apps.govdoc.manager.impl;

import com.seeyon.apps.govdoc.dao.GovdocDaiLuDao;
import com.seeyon.apps.govdoc.manager.GovdocLeaderAndPermissionRelationManager;
import com.seeyon.apps.govdoc.po.GovdocLeaderPermissionRelation;
import com.seeyon.apps.govdoc.vo.GovdocDaiLuVo;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.config.manager.ConfigManager;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.permission.manager.PermissionManager;
import com.seeyon.ctp.common.permission.vo.PermissionVO;
import com.seeyon.ctp.common.po.config.ConfigItem;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.FlipInfo;
import com.seeyon.ctp.util.annotation.AjaxAccess;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import org.apache.logging.log4j.util.Strings;

public class GovdocLeaderAndPermissionRelationManagerImpl implements GovdocLeaderAndPermissionRelationManager {
    private GovdocDaiLuDao govdocDaiLuDao;
    private OrgManager orgManager;
    private ConfigManager configManager;
    private PermissionManager permissionManager;

    public void setConfigManager(ConfigManager configManager) {
        this.configManager = configManager;
    }

    public void setOrgManager(OrgManager orgManager) {
        this.orgManager = orgManager;
    }

    public void setGovdocDaiLuDao(GovdocDaiLuDao govdocDaiLuDao) {
        this.govdocDaiLuDao = govdocDaiLuDao;
    }
    
    public void setPermissionManager(PermissionManager permissionManager) {
        this.permissionManager = permissionManager;
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    @AjaxAccess
    @Override
    public FlipInfo findLeaderAndPermissionRelation(FlipInfo flipInfo, Map params ) throws BusinessException {
        Long orgAccountId = AppContext.getCurrentUser().getAccountId();
        params.put("accountId", orgAccountId);
        flipInfo = govdocDaiLuDao.findLeaderAndPermissionRelation(flipInfo,params);
        List<GovdocDaiLuVo> list = new ArrayList<GovdocDaiLuVo>();
        String categoryName = "";
        String leaderName = "";
        
        String permissionRangeDid = "";
        String permissionRangeRec = "";
        String permissionRangeSig = "";
        if(!flipInfo.getData().isEmpty()){
            for (int i=0 ; i<flipInfo.getData().size() ; i++){
                GovdocDaiLuVo  daiLu = new GovdocDaiLuVo();
                GovdocLeaderPermissionRelation relation = (GovdocLeaderPermissionRelation) flipInfo.getData().get(i);
                daiLu.setId(relation.getId());
                daiLu.setLeaderId(relation.getLeaderId());
                leaderName = LeaderIdStrTransName(relation.getLeaderId());
                daiLu.setLeaderName(leaderName);
				/*
				 * daiLu.setCategoryId(relation.getCategory()); switch (relation.getCategory()){
				 * case 0 : categoryName="发文";break; case 1 : categoryName="收文";break; case 2 :
				 * categoryName="签报";break; default: categoryName=""; }
				 * daiLu.setCategoryName(categoryName);
				 * daiLu.setPermissionId(relation.getPermissionId()); permissionName =
				 * permissionIdStrTransName(relation.getPermissionId());
				 * daiLu.setPermissionName(permissionName);
				 */
                
                daiLu.setPermissionRangeDidID(relation.getPermissionRangeDid());
                permissionRangeDid = permissionIdStrTransName(relation.getPermissionRangeDid());
                daiLu.setPermissionRangeDidName(permissionRangeDid);
                
                daiLu.setPermissionRangeRecId(relation.getPermissionRangeRec());
                permissionRangeRec = permissionIdStrTransName(relation.getPermissionRangeRec());
                daiLu.setPermissionRangeRecName(permissionRangeRec);
                
                daiLu.setPermissionRangeSigId(relation.getPermissionRangeSig());
                permissionRangeSig = permissionIdStrTransName(relation.getPermissionRangeSig());
                daiLu.setPermissionRangeSigName(permissionRangeSig);
                list.add(daiLu);
            }
        }
        flipInfo.setData(list);
        return flipInfo;
    }

    @Override
    public FlipInfo findById(FlipInfo flipInfo, Map params) throws BusinessException {
        if(!params.isEmpty()){
            flipInfo.setData(govdocDaiLuDao.findById(Long.valueOf(params.get("id").toString())));
        }
        return flipInfo;
    }

    @Override
    public String LeaderIdStrTransName(String str) throws BusinessException {
        V3xOrgMember v3xOrgMembers = orgManager.getMemberById(Long.valueOf(str));
        return  v3xOrgMembers.getName();
    }

    @Override
    public String permissionIdStrTransName(String str) throws BusinessException {
        String permissionName = "";
        if(Strings.isNotBlank(str)){
            String [] permissions = str.split(",");
            for (int k = 0 ; k< permissions.length ; k++ ) {
                ConfigItem item =  configManager.getConfigItem(Long.valueOf(permissions[k]));
                if(("node.policy."+item.getConfigItem()).equals(ResourceUtil.getString("node.policy."+item.getConfigItem()))){
                    permissionName = permissionName + item.getConfigItem();
                    if( k < permissions.length -1){
                        permissionName  = permissionName + "、";
                    }
                }else{
                    permissionName = permissionName + ResourceUtil.getString("node.policy."+item.getConfigItem());
                    if( k < permissions.length -1){
                        permissionName  = permissionName + "、";
                    }
                }
            }
        }
        return permissionName;
    }

    @Override
    @AjaxAccess
    public int check(Object object1, Object object2, Object object3) throws BusinessException {
        String orgAccountId = object1.toString();
        String leaderId = object2.toString();
        String category = object3.toString();
        int length = govdocDaiLuDao.check(orgAccountId,leaderId,category);
        if(length > 0){
            return 1;
        }else{
            return 0;
        }
    }
    
    @Override
    @AjaxAccess
    public int checkNew(Object object1, Object object2) throws BusinessException {
        String orgAccountId = object1.toString();
        String leaderId = object2.toString();
        //String category = object3.toString();
        int length = govdocDaiLuDao.checkNew(orgAccountId,leaderId);
        if(length > 0){
            return 1;
        }else{
            return 0;
        }
    }

    @Override
    public boolean deleteLPRByLeaderId(String leaderId, Long orgAccountId, int category) throws BusinessException {
        boolean flag = govdocDaiLuDao.delete(leaderId,orgAccountId,category);
        if(flag){
            return true;
        }else {
            return false;
        }
    }
    
    @Override
    public boolean deleteLPRByLeaderNewId(String leaderId, Long orgAccountId) throws BusinessException {
        boolean flag = govdocDaiLuDao.deleteNew(leaderId,orgAccountId);
        if(flag){
            return true;
        }else {
            return false;
        }
    }

    @Override
    public void saveOrEdit(GovdocLeaderPermissionRelation govdocLPR) throws BusinessException {
        govdocDaiLuDao.saveOrEdit(govdocLPR);
    }

    @Override
    @AjaxAccess
    public void deleteList(String ids) throws BusinessException {
        String[] idss =  ids.split(",");
        for (String id:idss) {
            govdocDaiLuDao.deleteById(Long.valueOf(id));
        }
    }

	@Override
	@AjaxAccess
	public List<PermissionVO> getPermissionList(String leaderId,String accountId, String govdocType) throws BusinessException {
		List<PermissionVO> list = new ArrayList<PermissionVO>();
		if(Strings.isBlank(leaderId)||Strings.isBlank(govdocType)) {
			return list;
		}
		Long orgAccountId = Long.valueOf(accountId);
		Integer category1 = getCategory(govdocType);
		String category = "edoc_new_send_permission_policy";
		//根据参数判断类型权限
	    if(category1 == 0) {
	        category = "edoc_new_send_permission_policy";
	    }else if(category1 == 1){
	        category = "edoc_new_rec_permission_policy";
	    }else if(category1 == 2){
	        category = "edoc_new_qianbao_permission_policy";

	    }
		List<GovdocLeaderPermissionRelation> leaderPermission = govdocDaiLuDao.findByIdAndOrgAccountId(leaderId,Long.valueOf(orgAccountId),category1);
		for(GovdocLeaderPermissionRelation relation:leaderPermission) {
			String[] permissionIds = new String[0];
			String permissionRangeDid = relation.getPermissionRangeDid();
			String permissionRangeRec = relation.getPermissionRangeRec();
			String permissionRangeSig = relation.getPermissionRangeSig();
			if(category1 == 0&&Strings.isNotBlank(permissionRangeDid)) {
				permissionIds = permissionRangeDid.split(",");
			}else if(category1 == 1&&Strings.isNotBlank(permissionRangeRec)) {
				permissionIds = permissionRangeRec.split(",");
			}else if(category1 == 4&&Strings.isNotBlank(permissionRangeSig)) {
				permissionIds = permissionRangeSig.split(",");
			}
			
			for(String id : permissionIds) {
				PermissionVO permission = permissionManager.getPermission(Long.valueOf(id));
				if(permission.getBasicOperation().contains("Distribute")||permission.getCommonOperation().contains("JointlyIssued")||permission.getAdvancedOperation().contains("JointlyIssued")) {
					continue;
				}
				list.add(permission);
			}

		}
		if(list.isEmpty()) {
			List<PermissionVO> pList = permissionManager.getPermission(category, 1, orgAccountId);
			for(PermissionVO permission : pList) {
				if((permission.getBasicOperation().contains("Distribute")||permission.getCommonOperation().contains("JointlyIssued")||permission.getAdvancedOperation().contains("JointlyIssued"))) {
					continue;
				}
				if(permission.getLocation() == 0) {
					continue;
				}
				list.add(permission);
			}
		}
		list.sort(new Comparator<PermissionVO>() {

			@Override
			public int compare(PermissionVO o1, PermissionVO o2) {
				if(o1.getSort()>o2.getSort()) {
					return 1;
				}else {
					return -1;
				}
			}
			
		});
		return list;
	}
	
	private Integer getCategory(String govdocType) {
		if("1".equals(govdocType)) {
			return 0;
		}else if("2".equals(govdocType)) {
			return 1;
		}else if("4".equals(govdocType)) {
			return 2;
		}
		return 0;
	}

	@Override
	public GovdocLeaderPermissionRelation getById(Long id) throws BusinessException {
		return govdocDaiLuDao.get(id);
	}
}
