package com.seeyon.apps.govdoc.manager.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.seeyon.apps.agent.bo.AgentModel;
import com.seeyon.apps.agent.bo.MemberAgentBean;
import com.seeyon.apps.edoc.constants.EdocConstant.flowState;
import com.seeyon.apps.govdoc.constant.GovdocEnum;
import com.seeyon.apps.govdoc.constant.GovdocEnum.OldExchangeNodePolicyEnum;
import com.seeyon.apps.govdoc.constant.GovdocListEnum.GovdocListTypeEnum;
import com.seeyon.apps.govdoc.dao.GovdocListDao;
import com.seeyon.apps.govdoc.helper.GovdocHelper;
import com.seeyon.apps.govdoc.helper.GovdocListHelper;
import com.seeyon.apps.govdoc.manager.*;
import com.seeyon.apps.govdoc.util.GovdocUtil;
import com.seeyon.apps.govdoc.vo.GovdocListVO;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.affair.util.WFComponentUtil;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.ctpenumnew.EnumNameEnum;
import com.seeyon.ctp.common.ctpenumnew.manager.EnumManager;
import com.seeyon.ctp.common.dump.vo.DumpDataVO;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.permission.bo.Permission;
import com.seeyon.ctp.common.permission.manager.PermissionManager;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.po.ctpenumnew.CtpEnumBean;
import com.seeyon.ctp.common.po.ctpenumnew.CtpEnumItem;
import com.seeyon.ctp.common.workflowmanage.vo.WorkflowData;
import com.seeyon.ctp.organization.bo.V3xOrgAccount;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.DBAgent;
import com.seeyon.ctp.util.FlipInfo;
import com.seeyon.ctp.util.IdentifierUtil;
import com.seeyon.ctp.util.Strings;
import com.seeyon.v3x.edoc.domain.EdocSummary;
import com.seeyon.v3x.edoc.domain.EdocSummaryExtend;
import com.seeyon.v3x.edoc.manager.EdocHelper;
import com.seeyon.v3x.edoc.manager.EdocSummaryExtendManager;
import com.seeyon.v3x.edoc.manager.EdocSummaryManager;
import org.apache.commons.lang.math.NumberUtils;

/**
 * 新公文列表管理类
 *
 * @author 唐桂林
 */
public class GovdocListManagerImpl implements GovdocListManager {

    private static final String startUserNameIsNotExists = "startUserNameIsNotExists";
    private static final String sendAccountNameIsNotExists = "sendAccountNameIsNotExists";

    private EdocSummaryManager edocSummaryManager;
    private GovdocListDao govdocListDao;
    private GovdocListConfigManager govdocListConfigManager;
    private OrgManager orgManager;
    private EnumManager enumManagerNew;
    private PermissionManager permissionManager;
    private AffairManager affairManager;
    private GovdocOpenManager govdocOpenManager;
	private GovdocSummaryManager govdocSummaryManager;
    private GovdocPubManager govdocPubManager;
    private EdocSummaryExtendManager edocSummaryExtendManager;

    public EdocSummaryExtendManager getEdocSummaryExtendManager() {
		return edocSummaryExtendManager;
	}

	public void setEdocSummaryExtendManager(EdocSummaryExtendManager edocSummaryExtendManager) {
		this.edocSummaryExtendManager = edocSummaryExtendManager;
	}

	public void setGovdocPubManager(GovdocPubManager govdocPubManager) {
        this.govdocPubManager = govdocPubManager;
    }

    public void setGovdocOpenManager(GovdocOpenManager govdocOpenManager) {
		this.govdocOpenManager = govdocOpenManager;
	}

	public void setAffairManager(AffairManager affairManager) {
        this.affairManager = affairManager;
    }

    @Override
    public FlipInfo findPendingList(FlipInfo flipInfo, Map<String, String> condition) throws BusinessException {
        condition = setGovdocListCondition(condition);
		if (startUserNameIsNotExists.equals(condition.get("startUserId"))) {
			return new FlipInfo(); 
		}
		condition = getGovdocAgentInfo(condition);
		String listType = condition.get("listType");
		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		if ("listExchangeSendPending".equals(listType)) {
		    voList = govdocListDao.findSendPendingList(flipInfo, condition);
		}else if ("listExchangeSignPending".equals(listType)) {
		    voList = govdocListDao.findSignPendingList(flipInfo, condition);
		} else{
			voList = govdocListDao.findList(flipInfo, condition);
		}
		if(flipInfo == null) {
			flipInfo = new FlipInfo();
		}
		if (condition.containsKey("onlyCount")){//如果只统计count
			return flipInfo;
		}
		voList = setListVO(voList, condition);
		flipInfo.setData(voList);
		return flipInfo;	
	}
	
	@Override
	public int getPendingTotalCount(Map<String, String> condition)
			throws BusinessException {
		condition = setGovdocListCondition(condition);
		if (startUserNameIsNotExists.equals(condition.get("startUserId"))) {
			return 0; 
		}
		condition = getGovdocAgentInfo(condition);
		String listType = condition.get("listType");
		if ("listExchangeSendPending".equals(listType)) {
			return govdocListDao.findSendPendingListCount(condition);
		}else if ("listExchangeSignPending".equals(listType)) {
			return govdocListDao.findSignPendingListCount(condition);
		}else{
			return govdocListDao.findListCount(condition);
		}
	}

	@Override
	public FlipInfo findDoneList(FlipInfo flipInfo, Map<String, String> condition) throws BusinessException {
		condition = setGovdocListCondition(condition);
		if (startUserNameIsNotExists.equals(condition.get("startUserId"))) {
			return new FlipInfo(); 
		}
		if (sendAccountNameIsNotExists.equals(condition.get("exchangeSendUnitId"))) {
			return new FlipInfo(); 
		}
		condition = getGovdocAgentInfo(condition);
		String listType = condition.get("listType");
		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
        if ("listExchangeSendDone".equals(listType)) {
			voList = govdocListDao.findSendDoneList(flipInfo, condition);
		}else if ("listExchangeSignDone".equals(listType)) {
			voList = govdocListDao.findSignDoneList(flipInfo, condition);
		}else if ("listExchangeFallback".equals(listType)) {
			voList = govdocListDao.findFallbackList(flipInfo, condition);
		}
        else if ("listExchangeReturnedFallback".equals(listType)) {
            voList = govdocListDao.findReturnedFallbackList(flipInfo, condition);
        }
		else if (listType.startsWith("listExchangeCompeteDone")) {
			voList = govdocListDao.findExchangeCompeteDoneList(flipInfo, condition);
        }else if (listType.startsWith("listCompeteDoneOther") || listType.startsWith("listCompeteDoneAll")) {
            voList = govdocListDao.findCompeteDoneList(flipInfo, condition);
		} else{
			if(condition.containsKey("dumpData") && DumpDataVO.dataType.dumpData.getKey().equals(condition.get("dumpData"))){
				GovdocListDao govdocListDaoFK = (GovdocListDao)AppContext.getBean("govdocListDaoFK");
				if(govdocListDaoFK != null){
					voList = govdocListDaoFK.findList(flipInfo, condition);
				}
			}else{
				voList = govdocListDao.findList(flipInfo, condition);
			}
		}
		voList = setListVO(voList, condition);
		if(flipInfo == null) {
			flipInfo = new FlipInfo();
		}
		flipInfo.setData(voList);
		return flipInfo;
	}

	@Override
	public FlipInfo findCompeteDoneList(FlipInfo flipInfo, Map<String, String> condition) throws BusinessException {
		
		if(!govdocOpenManager.isGovCompeteExec()){
			return flipInfo;
		}
		condition = setGovdocListCondition(condition);
		if (startUserNameIsNotExists.equals(condition.get("startUserId"))) {
			return new FlipInfo(); 
		}
		if (sendAccountNameIsNotExists.equals(condition.get("exchangeSendUnitId"))) {
			return new FlipInfo(); 
		}
		condition = getGovdocAgentInfo(condition);
		List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
		voList = govdocListDao.findCompeteDoneList(flipInfo, condition);
		voList = setListVO(voList, condition);
		if(flipInfo == null) {
			flipInfo = new FlipInfo();
		}
		flipInfo.setData(voList);
		return flipInfo;
	}

    @Override
    public FlipInfo findSentList(FlipInfo flipInfo, Map<String, String> condition) throws BusinessException {
        condition = setGovdocListCondition(condition);
        if (startUserNameIsNotExists.equals(condition.get("startUserId"))) {
            return new FlipInfo();
        }
        condition = getGovdocAgentInfo(condition);
        List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
        if (condition.containsKey("dumpData") && DumpDataVO.dataType.dumpData.getKey().equals(condition.get("dumpData"))) {
            GovdocListDao govdocListDaoFK = (GovdocListDao) AppContext.getBean("govdocListDaoFK");
            if (govdocListDaoFK != null) {
                voList = govdocListDaoFK.findList(flipInfo, condition);
            }
        } else {
            voList = govdocListDao.findList(flipInfo, condition);
        }
        voList = setListVO(voList, condition);
        flipInfo.setData(voList);
        return flipInfo;
    }

    @Override
    public FlipInfo findWaitSendList(FlipInfo flipInfo, Map<String, String> condition) throws BusinessException {
        condition = setGovdocListCondition(condition);
        if (startUserNameIsNotExists.equals(condition.get("startUserId"))) {
            return new FlipInfo();
        }
        condition = getGovdocAgentInfo(condition);
        List<GovdocListVO> voList = govdocListDao.findList(flipInfo, condition);
        voList = setListVO(voList, condition);
        if (flipInfo == null) {
            flipInfo = new FlipInfo();
        }
        flipInfo.setData(voList);
        return flipInfo;
    }

    @Override
    public FlipInfo find4QuoteList(FlipInfo flipInfo, Map<String, String> condition) throws BusinessException {
        condition = setGovdocListCondition(condition);
        if (startUserNameIsNotExists.equals(condition.get("startUserId"))) {
            return new FlipInfo();
        }
        List<GovdocListVO> voList = govdocListDao.find4QuoteList(flipInfo, condition);
        voList = setListVO(voList, condition);
        flipInfo.setData(voList);
        return flipInfo;
    }

    @Override
    public FlipInfo findQueryResultList(FlipInfo flipInfo, Map<String, String> condition) throws BusinessException {
        condition = setGovdocListCondition(condition);
        if (startUserNameIsNotExists.equals(condition.get("startUserId"))) {
            return new FlipInfo();
        }
        List<GovdocListVO> voList = new ArrayList<GovdocListVO>();
        if (condition.containsKey("dataType") && DumpDataVO.dataType.dumpData.getKey().equals(String.valueOf(condition.get("dataType"))) && AppContext.hasPlugin("fk")) {
            GovdocListDao govdocListDaoFK = (GovdocListDao) AppContext.getBean("govdocListDaoFK");
            if (govdocListDaoFK != null) {
                voList = govdocListDaoFK.findQueryResultList(flipInfo, condition);
            }
        } else {
            voList = govdocListDao.findQueryResultList(flipInfo, condition);
        }
     
        //公文查询统计在查询条件中未包含扩展表的情况下，在显示中包含扩展表list元素的情况，需要单独处理查询公文扩展表
        String isNeedSearchSummaryExtendOnlyInDisplay = condition.get("isNeedSearchSummaryExtendOnlyInDisplay");
        String isNeedSearchSummaryExtend = condition.get("isNeedSearchSummaryExtend");
        if("true".equals(isNeedSearchSummaryExtendOnlyInDisplay) && "false".equals(isNeedSearchSummaryExtend)){
        	for(int i=0;i<voList.size();i++){
           	GovdocListVO vo = voList.get(i);
            	EdocSummaryExtend summaryExtend = edocSummaryExtendManager.getEdocSummaryExtend(vo.getSummaryId());
            	if(summaryExtend != null){
            		vo = EdocHelper.addEdocSummaryExtendObject(vo, summaryExtend);
            	}
            }
        }
        voList = setListVO(voList, condition);
        if (flipInfo.getSize() == -1) {
            flipInfo.setSize(voList.size());
        }
        DBAgent.memoryPaging(voList, flipInfo);
        
        //公文查询条件保存
        if (condition.containsKey("searchCustomType")) {
            String result = govdocOpenManager.saveEdocSearchAndStatCustomCondition(condition.get("searchCustomType"), 
            		AppContext.currentUserId(), 
            		AppContext.currentAccountId(),
            		condition.get("conditionColumns"),
            		condition.get("conditionColumnLabels"),
            		condition.get("customColumns"),
            		condition.get("customDisplayColumns"));
		}
        
        return flipInfo;
    }

    @Override
    public FlipInfo findSendRegisterList(FlipInfo flipInfo, Map<String, String> condition) throws BusinessException {
        condition = setGovdocListCondition(condition);
        if (startUserNameIsNotExists.equals(condition.get("startUserId"))) {
            return new FlipInfo();
        }
        List<GovdocListVO> voList = govdocListDao.findRegisterList(flipInfo, condition);
        //公文查询统计在查询条件中未包含扩展表的情况下，在显示中包含扩展表list元素的情况，需要单独处理查询公文扩展表
        String isNeedSearchSummaryExtendOnlyInDisplay = condition.get("isNeedSearchSummaryExtendOnlyInDisplay");
        String isNeedSearchSummaryExtend = condition.get("isNeedSearchSummaryExtend");
        if("true".equals(isNeedSearchSummaryExtendOnlyInDisplay) && "false".equals(isNeedSearchSummaryExtend)){
        	for(int i=0;i<voList.size();i++){
           	GovdocListVO vo = voList.get(i);
            	EdocSummaryExtend summaryExtend = edocSummaryExtendManager.getEdocSummaryExtend(vo.getSummaryId());
            	if(summaryExtend != null){
            		vo = EdocHelper.addEdocSummaryExtendObject(vo, summaryExtend);
            	}
            }
        }
        voList = setListVO(voList, condition);
        if (flipInfo == null) {
            flipInfo = new FlipInfo();
        }
        flipInfo.setData(voList);
        //查询条件保存
        if (condition.containsKey("searchCustomType")) {
            String result = govdocOpenManager.saveEdocSearchAndStatCustomCondition(condition.get("searchCustomType"), 
            		AppContext.currentUserId(), 
            		AppContext.currentAccountId(),
            		condition.get("conditionColumns"),
            		condition.get("conditionColumnLabels"),
            		condition.get("customColumns"),
            		condition.get("customDisplayColumns"));
		}
        return flipInfo;
    }

    @Override
    public FlipInfo findSignRegisterList(FlipInfo flipInfo, Map<String, String> condition) throws BusinessException {
        condition = setGovdocListCondition(condition);
        if (startUserNameIsNotExists.equals(condition.get("startUserId"))) {
            return new FlipInfo();
        }
        List<GovdocListVO> voList = govdocListDao.findRegisterList(flipInfo, condition);
        voList = setListVO(voList, condition);
        if (flipInfo == null) {
            flipInfo = new FlipInfo();
        }
        //公文查询统计在查询条件中未包含扩展表的情况下，在显示中包含扩展表list元素的情况，需要单独处理查询公文扩展表
        String isNeedSearchSummaryExtendOnlyInDisplay = condition.get("isNeedSearchSummaryExtendOnlyInDisplay");
        String isNeedSearchSummaryExtend = condition.get("isNeedSearchSummaryExtend");
        if("true".equals(isNeedSearchSummaryExtendOnlyInDisplay) && "false".equals(isNeedSearchSummaryExtend)){
        	for(int i=0;i<voList.size();i++){
           	GovdocListVO vo = voList.get(i);
            	EdocSummaryExtend summaryExtend = edocSummaryExtendManager.getEdocSummaryExtend(vo.getSummaryId());
            	if(summaryExtend != null){
            		vo = EdocHelper.addEdocSummaryExtendObject(vo, summaryExtend);
            	}
            }
        }
        flipInfo.setData(voList);
        //查询条件保存
        if (condition.containsKey("searchCustomType")) {
            String result = govdocOpenManager.saveEdocSearchAndStatCustomCondition(condition.get("searchCustomType"), 
            		AppContext.currentUserId(), 
            		AppContext.currentAccountId(),
            		condition.get("conditionColumns"),
            		condition.get("conditionColumnLabels"),
            		condition.get("customColumns"),
            		condition.get("customDisplayColumns"));
		}
        return flipInfo;
    }

    @Override
    public FlipInfo findRecRegisterList(FlipInfo flipInfo, Map<String, String> condition) throws BusinessException {
        condition = setGovdocListCondition(condition);
        if (startUserNameIsNotExists.equals(condition.get("startUserId"))) {
            return new FlipInfo();
        }
        List<GovdocListVO> voList = govdocListDao.findRegisterList(flipInfo, condition);
      //公文查询统计在查询条件中未包含扩展表的情况下，在显示中包含扩展表list元素的情况，需要单独处理查询公文扩展表
        String isNeedSearchSummaryExtendOnlyInDisplay = condition.get("isNeedSearchSummaryExtendOnlyInDisplay");
        String isNeedSearchSummaryExtend = condition.get("isNeedSearchSummaryExtend");
        if("true".equals(isNeedSearchSummaryExtendOnlyInDisplay) && "false".equals(isNeedSearchSummaryExtend)){
        	for(int i=0;i<voList.size();i++){
           	GovdocListVO vo = voList.get(i);
            	EdocSummaryExtend summaryExtend = edocSummaryExtendManager.getEdocSummaryExtend(vo.getSummaryId());
            	if(summaryExtend != null){
            		vo = EdocHelper.addEdocSummaryExtendObject(vo, summaryExtend);
            	}
            }
        }
        voList = setListVO(voList, condition);
        if (flipInfo == null) {
            flipInfo = new FlipInfo();
        }
        flipInfo.setData(voList);
        //查询条件保存
        if (condition.containsKey("searchCustomType")) {
            String result = govdocOpenManager.saveEdocSearchAndStatCustomCondition(condition.get("searchCustomType"), 
            		AppContext.currentUserId(), 
            		AppContext.currentAccountId(),
            		condition.get("conditionColumns"),
            		condition.get("conditionColumnLabels"),
            		condition.get("customColumns"),
            		condition.get("customDisplayColumns"));
		}
        return flipInfo;
    }

    /**
     * 设置查询条件
     *
     * @param type
     * @param condition
     * @return
     */
    private Map<String, String> setGovdocListCondition(Map<String, String> condition) throws BusinessException {
        for (String key : condition.keySet()) {
            if (Strings.isNotBlank(key) && key.indexOf("from_") >= 0) {
                String curKey = key.replace("from_", "");
                if (Strings.isNotBlank(key) && Strings.isNotBlank(condition.get(key)) && Strings.isBlank(condition.get(curKey))) {
                    condition.put(curKey, condition.get(key));
                }
            }
        }
        String listType = condition.get("listType");
        if (Strings.isBlank(condition.get("memberId"))) {
            condition.put("memberId", String.valueOf(AppContext.currentUserId()));
        }
        if (Strings.isBlank(condition.get("orgAccountId"))) {
            condition.put("orgAccountId", String.valueOf(AppContext.currentAccountId()));
        }
        if (Strings.isBlank(condition.get("departmentId"))) {
            condition.put("departmentId", String.valueOf(AppContext.getCurrentUser().getDepartmentId()));
        }
        if (Strings.isBlank(condition.get("needTotal"))) {
            condition.put("needTotal", "true");
        }
		if(Strings.isBlank(condition.get("deduplication"))) {
			condition.put("deduplication", "false");
		}
        //他人已办默认勾选显示最后一条
        if (condition.get("listType").startsWith("listCompeteDoneAll") ||
                condition.get("listType").startsWith("listCompeteDoneOther")) {
        	if (Strings.isBlank(condition.get("deduplication"))) {
        		condition.put("deduplication", "true");
        	}
        }

        if (listType.endsWith("Register")) {//公文登记簿，若govdocType为空，则赋值
            condition.put("govdocType", GovdocListTypeEnum.getGovdocTypeName(listType));
        }

        // 在这里， 用模糊查询， 把发起人的id都查询出来，  然后最后拼接hql，匹配
        if (Strings.isNotBlank(condition.get("startUserName"))) {//按发起人查询
            StringBuilder startUserIds = new StringBuilder("");
            List<V3xOrgMember> senderList = orgManager.getMemberByIndistinctName(condition.get("startUserName"));
            int i = 0;//OA-191840
            for (V3xOrgMember sender : senderList) {
                if (i < 999) {
                    if (startUserIds.length() > 0) {
                        startUserIds.append(",");
                    }
                    startUserIds.append(sender.getId());
                }
                i++;
            }
            condition.remove("startUserName");
            if (startUserIds.length() == 0) {
                condition.put("startUserId", startUserNameIsNotExists);
            } else {
                condition.put("startUserId", startUserIds.toString());
            }
        }

        if (Strings.isNotBlank(condition.get("configId"))) {//列表分类自定义配置
            if (govdocListConfigManager == null) {
                govdocListConfigManager = (GovdocListConfigManager) AppContext.getBean("govdocListConfigManager");
            }
            String nodePolicy = govdocListConfigManager.getNodePolicyByConfigId(Long.parseLong(condition.get("configId")));
            if (Strings.isNotBlank(nodePolicy)) {
                condition.put("nodePolicy", nodePolicy);
            }
        } else {
            String nodePolicy = GovdocListTypeEnum.getNodePolicyName(listType);
            String notInNodePolicy = GovdocListTypeEnum.getNotInNodePolicyName(listType);
            if (Strings.isNotBlank(nodePolicy)) {
                condition.put("nodePolicy", nodePolicy);
                //待分送/待签收/已签收
                if (listType.equals(GovdocListTypeEnum.listExchangeSendPending.getKey()) || listType.equals(GovdocListTypeEnum.listExchangeSignPending.getKey()) || listType.equals(GovdocListTypeEnum.listExchangeSignDone.getKey())) {
                    Long orgAccountId = Long.valueOf(condition.get("orgAccountId"));
                    EnumNameEnum categoryEnum = null;
                    if (listType.equals(GovdocListTypeEnum.listExchangeSendPending.getKey())) {
                        categoryEnum = EnumNameEnum.edoc_new_send_permission_policy;
                    } else {
                        categoryEnum = EnumNameEnum.edoc_new_change_permission_policy;
                    }
                    String policys = "";
                    List<Permission> permissions = permissionManager.getPermissionsByCategory(categoryEnum.name(), orgAccountId);
                    if (Strings.isNotEmpty(permissions)) {
                        String[] nodePolicys = nodePolicy.split(",");
                        for (String policy : nodePolicys) {
                            if (OldExchangeNodePolicyEnum.oldfasong.name().equals(policy) || OldExchangeNodePolicyEnum.oldqianshou.name().equals(policy)) {
                                if (Strings.isNotBlank(policys)) {
                                    policys += ",";
                                }
                                policys += policy;
                                continue;
                            }
                            for (Permission bean : permissions) {
                                if (bean.getBasicOperation().contains(policy)) {
                                    if (Strings.isNotBlank(policys)) {
                                        policys += ",";
                                    }
                                    policys += bean.getName();
                                }
                            }
                        }
                    }
                    condition.put("nodePolicy", policys);
                }
            }

            if (Strings.isNotBlank(notInNodePolicy)) {
                condition.put("notInNodePolicy", notInNodePolicy);
            }
        }

        return condition;
    }

    /**
     * 代理人处理
     *
     * @param condition
     * @param userId
     * @return
     */
    @SuppressWarnings("unused")
    private Map<String, String> getGovdocAgentInfo(Map<String, String> condition) {
        Long memberId = Long.parseLong(condition.get("memberId"));
        //获取代理相关信息
        List<AgentModel> _agentModelList = MemberAgentBean.getInstance().getAgentModelList(memberId);
        List<AgentModel> _agentModelToList = MemberAgentBean.getInstance().getAgentModelToList(memberId);
        List<AgentModel> agentModelList = null;
        boolean agentToFlag = false;
        boolean agentFlag = false;
        if (_agentModelList != null && !_agentModelList.isEmpty()) {
            agentModelList = _agentModelList;
            agentFlag = true;
        } else if (_agentModelToList != null && !_agentModelToList.isEmpty()) {
            agentModelList = _agentModelToList;
            agentToFlag = true;
        }

        List<AgentModel> edocAgent = new ArrayList<AgentModel>();
        if (agentModelList != null && !agentModelList.isEmpty()) {
            java.util.Date now = new java.util.Date();
            for (AgentModel agentModel : agentModelList) {
                if ((agentModel.isHasEdocFree() || agentModel.isHasEdocTemplate()) && agentModel.getStartDate().before(now) && agentModel.getEndDate().after(now)) {
                    edocAgent.add(agentModel);
                }
            }
        }
        boolean isProxy = false;
        if (edocAgent != null && !edocAgent.isEmpty()) {
            isProxy = true;
        } else {
            agentFlag = false;
            agentToFlag = false;
        }
        /*condition.put("edocAgent", edocAgent);*/
        condition.put("agentToFlag", agentToFlag + "");
        condition.put("agentFlag", agentFlag + "");
        return condition;
    }

    /**
     * @param request
     * @param response
     * @param modelAndView
     * @param queryList
     * @param condition
     * @return
     */
    @SuppressWarnings("unchecked")
    private List<GovdocListVO> setListVO(List<GovdocListVO> voList, Map<String, String> condition) throws BusinessException {
        String listType = condition.get("listType");
        Long memberId = Long.parseLong(condition.get("memberId"));
        //boolean agentFlag = GovdocParamUtil.getBoolean(condition, "agentFlag") || GovdocParamUtil.getBoolean(condition, "agentToFlag");
        CtpEnumBean docType = enumManagerNew.getEnumByProCode(EnumNameEnum.edoc_doc_type.name());//公文种类
        CtpEnumBean sendType = enumManagerNew.getEnumByProCode(EnumNameEnum.edoc_send_type.name());//行文类型
        CtpEnumBean secretLevel = enumManagerNew.getEnumByProCode(EnumNameEnum.edoc_secret_level.name());//密级
        CtpEnumBean urgentLevel = enumManagerNew.getEnumByProCode(EnumNameEnum.edoc_urgent_level.name());//紧急
        CtpEnumBean unitLevel = enumManagerNew.getEnumByProCode(EnumNameEnum.edoc_unit_level.name());//公文类别
        CtpEnumBean keepPeriod = enumManagerNew.getEnumByProCode(EnumNameEnum.edoc_keep_period.name());//保存期限
        //性能提升，预归档路径缓存
        Map<String, String> docPathMap = new HashMap<String, String>();
        //性能提升，领导批示
        Map<Long, String> leaderPishiMap = new HashMap<Long, String>();
        List<Long> summaryIds = new ArrayList<Long>();
        for (GovdocListVO listVo : voList) {
            summaryIds.add(listVo.getSummaryId());
        }
        if (edocSummaryManager == null) {
            edocSummaryManager = (EdocSummaryManager) AppContext.getBean("edocSummaryManager");
        }
        for (GovdocListVO listVo : voList) {
            //分送状态判断

            if(listVo.getProcessId() != null && listVo.getExchangeSendAffairId() != null && listVo.getExchangeSendAffairId().longValue() != -1){

                if (listVo.getExchangeSendAffairId().equals(listVo.getAffairId())){

                    listVo.setCompeteDoneType(null);
                }
                else
                {
                    CtpAffair sendAffair = affairManager.get(listVo.getExchangeSendAffairId());
                    if(sendAffair!= null && sendAffair.getMemberId().equals(listVo.getAffairMemberId().longValue()) && "true".equals(condition.get("deduplication")))
                    {
                        listVo.setCompeteDoneType(null);
                    }
                    else
                    {
                        listVo.setCompeteDoneType("no");
                    }

                }
            }
            //附件区!分
            listVo.setHasAtt(IdentifierUtil.lookupInner(listVo.getIdentifier(), EdocSummary.INENTIFIER_INDEX.HAS_ATTACHMENTS.ordinal(), '1'));
            //发起人/拟稿人
            V3xOrgMember startUser = orgManager.getMemberById(listVo.getStartUserId());
            listVo.setStartUserName(startUser == null ? "" : GovdocUtil.getMemberName(startUser.getId()));
            //防护，防止登记簿过来时空指针异常,解决 GOVB-85 公文统计-收发文登记薄查询都报错
            //已办所属人
            if(listVo.getCompeteTransactorId()!=null && (listType.equals("listCompeteDoneAll") || Strings.isNotBlank(condition.get("competeDoneType")))) {
                if("faxing".equals(listVo.getAffairNodePolicy()) || govdocPubManager.isFensong(listVo.getAffairId(),listVo.getSummaryId()))
                {
                    //todo james "other图标控制"
                    if(Long.valueOf(listVo.getCompeteTransactorId()).equals(AppContext.getCurrentUser().getId())) {
                        listVo.setCompeteDoneType("");
                    } else {
                        listVo.setCompeteDoneType("other");
                    }
                }


            }

            if(listVo.getCompeteTransactorId()!=null)
            {
                listVo.setCompeteTransactor(GovdocUtil.getMemberName(Long.valueOf(listVo.getCompeteTransactorId())));
            }
            //当前待办人
            if (listVo.getSummaryState() != null && (listVo.getSummaryState() == flowState.terminate.ordinal() || listVo.getSummaryState() == flowState.finish.ordinal())) {
                listVo.setCurrentNodesInfo(ResourceUtil.getString("collaboration.list.finished.label"));
            } else {
                listVo.setCurrentNodesInfo(GovdocHelper.parseCurrentNodesInfo(listVo.getCompleteTime(), listVo.getCurrentNodesInfo(), Collections.EMPTY_MAP));
            }
            //流程期限
            listVo.setSummaryDeadLineName(GovdocHelper.getDeadLineName(listVo.getSummaryDeadlineDatetime()));
            //节点期限
            if (listVo.getAffairExpectedProcessTime() != null) {
                listVo.setAffairDeadLineName(WFComponentUtil.getDeadLineName(listVo.getAffairExpectedProcessTime()));
            } else {
                listVo.setAffairDeadLineName(WFComponentUtil.getDeadLineName(listVo.getAffairDeadlineDate()));
            }
            //公文统计流转状态state字段查询0-未结束 3-已结束时特殊处理  liusy
            String summaryState = condition.get("summaryState");
            String exchangeSendAffairId = condition.get("exchangeSendAffairId");
            if(Strings.isNotBlank(summaryState) && ("0".equals(summaryState)||"3".equals(summaryState))){//未结束、已结束
            	if (listVo.getSummaryState() != null) {
                    if (listVo.getSummaryState().intValue() == 0) {
                        listVo.setSummaryTransferStatusName(ResourceUtil.getString("govdoc.Notfinished"));
                    } else if (listVo.getSummaryState().intValue() == 3) {
                    	listVo.setSummaryTransferStatusName(ResourceUtil.getString("govdoc.Hasended"));
                    }
                }
            }else if(Strings.isNotBlank(exchangeSendAffairId) && "1".equals(exchangeSendAffairId)){//已分送
            	if (listVo.getSummaryState() != null) {
                    listVo.setSummaryTransferStatusName(ResourceUtil.getString("govdoc.sent"));
                }
            }
            else{
            	//设置交换状态(未结束/已结束/已分送)
                if (listVo.getSummaryTransferStatus() != null && listVo.getSummaryTransferStatus().intValue() >= 2) {
                    listVo.setSummaryTransferStatusName(ResourceUtil.getString(GovdocEnum.TransferStatus.getByKey(listVo.getSummaryTransferStatus()).getLabel()));
                } 
                else if (listVo.getSummaryState() != null) {
                    if (listVo.getSummaryState() != null) {
                        if (listVo.getSummaryState().intValue() == 0) {
                            listVo.setSummaryTransferStatusName(ResourceUtil.getString("govdoc.Notfinished"));
                        } else if (listVo.getSummaryState().intValue() == 1 || listVo.getSummaryState().intValue() == 3) {
                            if (listVo.getExchangeSendAffairId() == null || listVo.getExchangeSendAffairId() == -1) {
                                listVo.setSummaryTransferStatusName(ResourceUtil.getString("govdoc.Hasended"));
                            } else {
                                listVo.setSummaryTransferStatusName(ResourceUtil.getString("govdoc.sent"));
                            }
                        }
                    }
                }
            }
            
            //设置督办时间
            listVo.setSurplusTime(GovdocListHelper.calculateSurplusTime(listVo.getAffairReceiveTime(), listVo.getAffairExpectedProcessTime()));
            //公文枚举值设置
            GovdocListHelper.fillListEnumLabel(listVo, docType, sendType, keepPeriod, secretLevel, urgentLevel, unitLevel);
            //是否归档和归档路径
            if ("listQuery".equals(listType)) {
                GovdocListHelper.fillListArchiveData(listVo, docPathMap);
            }
            //列表展示动态标题
            if (Strings.isNotBlank(listVo.getAffairSubject())) {
                listVo.setSubject(listVo.getAffairSubject());
            }
            //自动发起标题
            if (listVo.getAutoRun() && !listVo.getSubject().contains("自动发起") ) {
                listVo.setSubject(ResourceUtil.getString("collaboration.newflow.fire.subject", listVo.getSubject()));
            }
            //todo james 显示控制
            if(listType.equals("listCompeteDoneAll") || listType.equals("listCompeteDoneOther") || Strings.isNotBlank(condition.get("competeDoneType"))) {
//                if(Strings.isNotBlank(condition.get("competeDoneType"))) {
//                    listVo.setCompeteDoneType(condition.get("competeDoneType"));
//                }
            } else { //设置代理文字
                if (listVo.getAffairMemberId() != null && memberId != null
                        && !"niwen".equals(listVo.getAffairNodePolicy())) {
                    GovdocListHelper.fillListAgent(listVo, true, memberId, listType);
                }
            }
            if (GovdocUtil.isExchangeSendPendingList(listType)) {
                // 性能问题，上一节点处理人使用preApprover
                // GovdocWorkflowHelper.fillLastProcessLog(listVo);
                if (listVo.getAffairPreApprover() != null) {
                    listVo.setPreUserName(orgManager.getMemberById(listVo.getAffairPreApprover()).getName());
                }
            } else if (GovdocUtil.isExchangeSignList(listType)) {
                if (listVo.getOrgAccountId() != null) {
                    V3xOrgAccount account = orgManager.getAccountById(listVo.getOrgAccountId());
                    if (account != null) {
                        listVo.setExchangeSendUnitName(account.getName());
                    }
                }
            }
            listVo.setLeaderCommondNo(GovdocListHelper.getAllLeaderPishi(leaderPishiMap, summaryIds, listVo.getSummaryId()));
            if (listVo.getAffairTrack() == null) {
                listVo.setAffairTrack(0);
            }
            
            //处理枚举显示
            if(listVo.getList1() != null){
            	listVo.setList1(getListValue(listVo.getList1()));
            }
            if(listVo.getList2() != null){
            	listVo.setList2(getListValue(listVo.getList2()));
            }
            if(listVo.getList3() != null){
            	listVo.setList3(getListValue(listVo.getList3()));
            }
            if(listVo.getList4() != null){
            	listVo.setList4(getListValue(listVo.getList4()));
            }
            if(listVo.getList5() != null){
            	listVo.setList5(getListValue(listVo.getList5()));
            }
            if(listVo.getList6() != null){
            	listVo.setList6(getListValue(listVo.getList6()));
            }
            if(listVo.getList7() != null){
            	listVo.setList7(getListValue(listVo.getList7()));
            }
            if(listVo.getList8() != null){
            	listVo.setList8(getListValue(listVo.getList8()));
            }
            if(listVo.getList9() != null){
            	listVo.setList9(getListValue(listVo.getList9()));
            }
            if(listVo.getList10() != null){
            	listVo.setList10(getListValue(listVo.getList10()));
            }
            if(listVo.getList11() != null){
            	listVo.setList11(getListValue(listVo.getList11()));
            }
            if(listVo.getList12() != null){
            	listVo.setList12(getListValue(listVo.getList12()));
            }
            if(listVo.getList13() != null){
            	listVo.setList13(getListValue(listVo.getList13()));
            }
            if(listVo.getList14() != null){
            	listVo.setList14(getListValue(listVo.getList14()));
            }
            if(listVo.getList15() != null){
            	listVo.setList15(getListValue(listVo.getList15()));
            }
            if(listVo.getList16() != null){
            	listVo.setList16(getListValue(listVo.getList16()));
            }
            if(listVo.getList17() != null){
            	listVo.setList17(getListValue(listVo.getList17()));
            }
            if(listVo.getList18() != null){
            	listVo.setList18(getListValue(listVo.getList18()));
            }
            if(listVo.getList19() != null){
            	listVo.setList19(getListValue(listVo.getList19()));
            }
            if(listVo.getList20() != null){
            	listVo.setList20(getListValue(listVo.getList20()));
            }
        }
        return voList;
    }

    private String getListValue(String listId)
    {
        if(NumberUtils.isNumber(listId)) {
            CtpEnumItem item = enumManagerNew.getEnumItem(Long.parseLong(listId));
            return item.getShowvalue();
        }
        else{
            return listId;
        }

    }

	@Override
	public List<WorkflowData> getAdminWfDataList(FlipInfo flipInfo, Map<String,Object> conditionParam,long accountId,boolean isPage,FlipInfo fi)  throws BusinessException {
		String currentNodesInfo = (String) conditionParam.get("currentNodesInfo");
		List<WorkflowData> result = null;
		
		if(Strings.isNotBlank(currentNodesInfo)) {
			
    		List<Long> objectIds =  (List<Long>) conditionParam.get("objectIds");
    		List<EdocSummary> edocSummary = govdocSummaryManager.findEdocSummarysByIds(objectIds);
    		
    		String flow = (String)conditionParam.get("flowstate");
            int flowstate = 0;
            if(Strings.isNotBlank(flow)){
                flowstate = Integer.parseInt(flow);
            }
    		result = buildResult(edocSummary,flowstate);
    	}else {
    		
    		result = govdocListDao.getAdminWfDataList(flipInfo, conditionParam, accountId, isPage,fi);
    		
    	}
    	
		return result;
		
		// return govdocListDao.getAdminWfDataList(flipInfo, conditionParam, accountId, isPage,fi);
	}
    @Override
    public Integer getRegisterCount(Map<String, String> condition) throws BusinessException {
        return govdocListDao.getRegisterCount(condition);
    }

    @Override
    public FlipInfo getList4Xiaoz(FlipInfo flipInfo, Map<String, String> condition) throws BusinessException {
        condition = setGovdocListCondition(condition);
        if (startUserNameIsNotExists.equals(condition.get("startUserId"))) {
            return new FlipInfo();
        }
        List<GovdocListVO> voList = govdocListDao.findList(flipInfo, condition);
        voList = setListVO(voList, condition);
        if (flipInfo == null) {
            flipInfo = new FlipInfo();
        }
        flipInfo.setData(voList);
        return flipInfo;
    }

	public List<WorkflowData> buildResult(List<EdocSummary> result, int flowstate)
			throws BusinessException {
		
        List<WorkflowData> list = new ArrayList<WorkflowData>();
        
        for(EdocSummary summary : result) {
        	WorkflowData flow = new WorkflowData();
        	
        	flow.setEdocType(-1);
        	flow.setSummaryId(String.valueOf(summary.getId()));
        	
        	//应用类型
        	int app = ApplicationCategoryEnum.edoc.getKey();
        	flow.setAppEnum(app);
        	
        	flow.setSubject(summary.getSubject());
        	
        	Long startUserId = summary.getStartUserId();
        	if(startUserId != null) {
        		V3xOrgMember member = null;
        		try {
        			member = orgManager.getMemberById(startUserId);
        		} catch (BusinessException e) {
        		}
        		if(member != null) {
        			flow.setInitiator(member.getName());
        		}
        	}
        	
        	flow.setSendTime(summary.getCreateTime());
        	flow.setProcessId(summary.getProcessId());
        	flow.setCaseId(summary.getCaseId());
        	flow.setDeadLine(summary.getDeadline());
        	flow.setAdvanceRemind(summary.getAdvanceRemind());
        	flow.setAccountId(summary.getOrgAccountId());
        	
        	Long templeteId = summary.getTempleteId();
        	if(templeteId!= null){
        		flow.setIsFromTemplete(true) ;
        		flow.setTempleteId(templeteId);
        	}
        	
        	GovdocUtil.setWorkflowDataByEdocSummary(summary, flow);
        	
        	Date deadlineDatetime=summary.getDeadlineDatetime();
        	//设置流程期限显示内容
        	flow.setDeadlineDatetimeName(WFComponentUtil.getDeadLineName(deadlineDatetime));
        	String currentNodesInfo = summary.getCurrentNodesInfo();
        	if(flowstate == flowState.terminate.ordinal() || flowstate == flowState.finish.ordinal()){
        		currentNodesInfo = "";
        	}else if(Strings.isNotBlank(currentNodesInfo) && currentNodesInfo.indexOf(";;") != -1){
        		currentNodesInfo = currentNodesInfo.replaceAll(";;", ";");
        	}
        	summary.setCurrentNodesInfo(currentNodesInfo);
        	flow.setCurrentNodesInfo(GovdocHelper.parseCurrentNodesInfo(summary));
        	flow.setSecretLevel(summary.getSecretLevel());
        	
        	flow.setFormAppId(summary.getFormAppid());
        	
        	flow.setFormRecordId(summary.getFormRecordid());
        	if(flowstate == 1 || flowstate == 3){
        		flow.setEndFlag(0); //0 - 结束
        	}
        	list.add(flow);
        	
        }
        
        return list;
    }
    /*************************** 99999 Spring注入，请将业务写在上面 start ******************************/
    public void setGovdocListDao(GovdocListDao govdocListDao) {
        this.govdocListDao = govdocListDao;
    }

    public void setOrgManager(OrgManager orgManager) {
        this.orgManager = orgManager;
    }

    public void setEnumManagerNew(EnumManager enumManagerNew) {
        this.enumManagerNew = enumManagerNew;
    }

    public void setGovdocListConfigManager(GovdocListConfigManager govdocListConfigManager) {
        this.govdocListConfigManager = govdocListConfigManager;
    }

    public void setPermissionManager(PermissionManager permissionManager) {
        this.permissionManager = permissionManager;
    }

    public void setEdocSummaryManager(EdocSummaryManager edocSummaryManager){
        this.edocSummaryManager = edocSummaryManager;
    }
	
	public void setGovdocSummaryManager(GovdocSummaryManager govdocSummaryManager) {
		this.govdocSummaryManager = govdocSummaryManager;
	}
    /*************************** 99999 Spring注入，请将业务写在上面   end ******************************/


}
