package com.seeyon.apps.govdoc.manager.impl;

import com.seeyon.apps.govdoc.manager.GovdocCommentManager;
import com.seeyon.apps.govdoc.manager.GovdocProcessManager;
import com.seeyon.apps.govdoc.vo.MemberPostVO;
import com.seeyon.ctp.common.ModuleType;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.content.comment.Comment;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.organization.bo.MemberPost;
import com.seeyon.ctp.organization.bo.V3xOrgDepartment;
import com.seeyon.ctp.organization.bo.V3xOrgAccount;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.bo.V3xOrgPost;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.workflow.exception.BPMException;
import com.seeyon.ctp.workflow.manager.ProcessManager;
import com.seeyon.ctp.workflow.po.ProcessInRunningDAO;
import net.joinwork.bpm.definition.*;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.*;

/**
 * Created by Seeyon on 2019-11-28.
 */
public class GovdocProcessManagerImpl implements GovdocProcessManager {
    private static final Log LOGGER = LogFactory.getLog(GovdocProcessManagerImpl.class);
    private ProcessManager processManager = null;
    private OrgManager orgManager;
    private AffairManager affairManager;
    private GovdocCommentManager govdocCommentManager;

    public void setGovdocCommentManager(GovdocCommentManager govdocCommentManager) {
        this.govdocCommentManager = govdocCommentManager;
    }

    public void setAffairManager(AffairManager affairManager) {
        this.affairManager = affairManager;
    }

    public void setOrgManager(OrgManager orgManager) {
        this.orgManager = orgManager;
    }

    public void setProcessManager(ProcessManager processManager) {
        this.processManager = processManager;
    }

    @Override
    public MemberPostVO getDefaultMemberPost(CtpAffair affair,V3xOrgMember currentMember) {
        Long activityId = affair.getActivityId();
        String processId = affair.getProcessId();

        if(activityId == null  || Strings.isBlank(processId))
        {
            return getMainMemberPost(currentMember);
        }


        try {
            ProcessInRunningDAO processInRunningDAO = processManager.getProcessInRunningDAO(processId, false);
            BPMProcess process = processInRunningDAO.getProcess();
            List<BPMTransition> links = process.getLinks();
            if(links == null || links.isEmpty())
            {
                return getMainMemberPost(currentMember);
            }
            //获取当前节点所在的流程节点
            BPMTransition targetBPMTransition = null;
            //转换links到map key to的ID
            Map<String,List<BPMTransition>> mapLink = new HashMap<String, List<BPMTransition>>();
            for(BPMTransition templink: links)
            {
                List<BPMTransition> listBPMTransition = mapLink.get(templink.getTo().getId());
                if(listBPMTransition == null)
                {
                    listBPMTransition = new ArrayList<BPMTransition>();
                    mapLink.put(templink.getTo().getId(),listBPMTransition);

                }

                listBPMTransition.add(templink);
            }

            //获取默认的落款单位部门信息
            List<MemberPostVO> memberPostList = null;
            try {
                memberPostList = getMemberPostList(currentMember);
            } catch (BusinessException e) {
                return getMainMemberPost(currentMember);
            }
            if(memberPostList == null || memberPostList.isEmpty())
            {
                return getMainMemberPost(currentMember);
            }

            if(memberPostList.size() == 1)
            {
                return memberPostList.get(0);
            }


            Map<String,List<Comment>>  mapComment = null;
            try {
                mapComment = managerComment(getAffairByEdocId(affair.getObjectId()),getCommmetByEdocId(affair.getObjectId()));
            } catch (BusinessException e) {
                LOGGER.error("获取公文评论报错",e);
                return getMainMemberPost(currentMember);
            }
            //当前处理是否有暂存代办或者处理过了，如果处理过了直接默认处理的落款
            if(mapComment != null && !mapComment.isEmpty())
            {
                //判断当前处理节点是否已将有落款了
                List<Comment> currentCommentList = mapComment.get(affair.getActivityId().toString());

                if(currentCommentList !=null && !currentCommentList.isEmpty())
                {
                    sortCommentListDesc(currentCommentList);
                    for(Comment currentComment : currentCommentList)
                    {

                        Long departId = currentComment.getDepartmentId();
                        if(departId != null && currentComment.getAffairId() != null && (affair.getId().longValue() == currentComment.getAffairId().longValue() || affair.getMemberId().longValue() == currentComment.getCreateId().longValue())){

                            for(MemberPostVO tempMemberPostVO: memberPostList)
                            {
                                Long memberDepartId = tempMemberPostVO.getDepId();
                                if(memberDepartId != null && memberDepartId.longValue() == departId.longValue())
                                {
                                    return tempMemberPostVO;
                                }
                            }

                        }
                    }
                }

            }
            Map<String,String>  usedActivityId  = new HashMap<String, String>();

            for(BPMTransition templink: links)
            {
                if(templink.getFrom().getId().equals(activityId.toString()))
                {
                    targetBPMTransition = templink;
                    break;
                }
            }
            //值班上级部门里的节点
            Collection<MemberPostVO> upMemberPost = filterDownDepart(memberPostList);
            Map<Long, V3xOrgDepartment> allDepartmentMap = getALLDepartmentMap(upMemberPost);
            MemberPostVO  defualtMemberVo= getPreMemberPostVO(targetBPMTransition, mapLink,mapComment,memberPostList,usedActivityId,allDepartmentMap);

            if(defualtMemberVo!= null)
            {
                return defualtMemberVo;
            }

            //流程发起节点
            if(process.getStart()!= null && process.getStart().getActorList() != null && !process.getStart().getActorList().isEmpty())
            {
                Object actor = process.getStart().getActorList().get(0);
                if(actor instanceof BPMActor && ((BPMActor) actor).getParty() != null)
                {
                     String startMemberId = ((BPMActor) actor).getParty().getId();
                     if(Strings.isNotBlank(startMemberId))
                     {
                         V3xOrgMember startMember = orgManager.getMemberById(Long.parseLong(startMemberId));
                         if(startMember != null)
                         {
                             List<MemberPostVO> startMemberPostList = getMemberPostList(startMember);
                             for(MemberPostVO startMemberPost :startMemberPostList)
                             {
                                 Long departId = startMemberPost.getDepId();
                                 for(MemberPostVO memberPost: memberPostList)
                                 {
                                     Long memberDepartId = memberPost.getDepId();
                                     if(memberDepartId != null && departId!= null && memberDepartId.longValue() == departId.longValue())
                                     {
                                         defualtMemberVo = memberPost;
                                         break;
                                     }
                                 }

                                 if(defualtMemberVo != null)
                                 {
                                     return defualtMemberVo;
                                 }

                                 defualtMemberVo = getParentDepartmentForOpinion(departId,memberPostList,allDepartmentMap);

                                 if(defualtMemberVo != null)
                                 {
                                     return defualtMemberVo;
                                 }
                             }


                         }


                     }
                }
            }

            if(defualtMemberVo == null)
            {
                return memberPostList.get(0);
            }
            return defualtMemberVo;
        } catch (Exception e) {
            LOGGER.error(e);
            return getMainMemberPost(currentMember);
        }


    }

    public  Map<String,List<Comment>>  managerComment(List<CtpAffair> affairList,List<Comment> commentList) {
        Map<String, List<Comment>> mapComment = new HashMap<String, List<Comment>>();
        if (affairList == null || affairList.isEmpty() || commentList == null || commentList.isEmpty()) {
            return mapComment;
        }
        for (Comment comment:commentList)
        {
            Long pid = comment.getPid();
            if(pid != null && pid == 0)
            {
                Long commentAffairId = comment.getAffairId();
                if(commentAffairId != null)
                {
                    for (CtpAffair affair : affairList) {
                        Long affairId = affair.getId();
                        if(affair.getActivityId() != null  && commentAffairId.longValue() == affairId)
                        {
                            List<Comment> list = mapComment.get(affair.getActivityId().toString());
                            if(list == null)
                            {
                                list = new ArrayList<Comment>();
                                mapComment.put(affair.getActivityId().toString(),list);
                            }
                            list.add(comment);
                            break;
                        }

                    }
                }
            }

        }
        return  mapComment;

    }
    public List<CtpAffair> getAffairByEdocId(Long edocId) throws BusinessException {
        List<CtpAffair> affairList = affairManager.getAffairs(edocId);
        return affairList;
    }
    public List<Comment> getCommmetByEdocId(Long edocId) throws BusinessException {
        List<Comment> commentList = govdocCommentManager.getCommentList(ModuleType.edoc, edocId);
        return commentList;
    }

    /**
     * 比对上一节点部门
     * @param targetBPMTransition
     * @param mapLink
     * @param mapComment
     * @param memberPostList
     * @param usedActivityIdMap
     * @return
     */
    private MemberPostVO getPreMemberPostVO(BPMTransition targetBPMTransition, Map<String, List<BPMTransition>> mapLink, Map<String,List<Comment>> mapComment,List<MemberPostVO> memberPostList,Map<String,String>  usedActivityIdMap,Map<Long, V3xOrgDepartment> allDepartmentMap) {
        //获取上一节点
        List<BPMTransition> preListLink = getPreNode(mapLink, targetBPMTransition);
        if(preListLink == null)
        {
            return null;
        }
        for(BPMTransition preTemp : preListLink)
        {
            String activityId = preTemp.getFrom().getId();
            if(usedActivityIdMap.get(activityId)!= null){
                return null;
            }
            usedActivityIdMap.put(activityId,activityId);
            //如果上一几点是路由节点继续获取上一级
            if(preTemp.getFrom() instanceof BPMStart)
            {
                return null;
            }

            if(preTemp.getFrom()instanceof BPMHumenActivity)
            {
                //获取默认落款
                String policyId = preTemp.getFrom().getSeeyonPolicy().getId();
                if("zhihui".equals(policyId))
                {
                    MemberPostVO memberPostVO = getPreMemberPostVO(preTemp, mapLink,mapComment,memberPostList,usedActivityIdMap,allDepartmentMap);
                    if(memberPostVO != null)
                    {
                        return memberPostVO;
                    }
                    else
                    {
                        continue;
                    }
                }
                List<Comment> edocCommentList = mapComment.get(activityId);


                if(edocCommentList == null || edocCommentList.isEmpty())
                {
                    MemberPostVO memberPostVO = getPreMemberPostVO(preTemp, mapLink,mapComment,memberPostList,usedActivityIdMap,allDepartmentMap);
                    if(memberPostVO != null)
                    {
                        return memberPostVO;
                    }
                }
                else
                {
                    MemberPostVO memberPostVO = null;
                    if(edocCommentList.size() > 1)
                    {
                        sortCommentListDesc(edocCommentList);
                    }


                    for(Comment edocComment : edocCommentList)
                    {
                        Long departId = edocComment.getDepartmentId();
                        if(departId != null)
                        {


                            for(MemberPostVO memberPost: memberPostList)
                            {
                                Long memberDepartId = memberPost.getDepId();
                                if(memberDepartId != null && memberDepartId.longValue() == departId.longValue())
                                {
                                    memberPostVO = memberPost;
                                    break;
                                }
                            }

                            if(memberPostVO != null)
                            {
                                return memberPostVO;
                            }

                        }

                    }

                    for(Comment edocComment : edocCommentList) {
                        Long departId = edocComment.getDepartmentId();
                        if(departId != null) {
                            memberPostVO = getParentDepartmentForOpinion(departId, memberPostList, allDepartmentMap);
                            if (memberPostVO != null) {
                                return memberPostVO;
                            }
                        }
                    }
                    // 如果没有继续
                    if(memberPostVO == null)
                    {

                        memberPostVO = getPreMemberPostVO(preTemp, mapLink,mapComment,memberPostList,usedActivityIdMap,allDepartmentMap);
                        if(memberPostVO != null)
                        {
                            return memberPostVO;

                        }
                    }
                }

            }
            else
            {
                // 递归获取上一节
                MemberPostVO memberPostVO = getPreMemberPostVO(preTemp, mapLink,mapComment,memberPostList,usedActivityIdMap,allDepartmentMap);
                if(memberPostVO != null)
                {
                    return memberPostVO;
                }
            }

        }
        return null;
    }

    /**
     * 获取上一节点
     * @param mapLink
     * @param targetBPMTransition
     * @return
     */
    public List<BPMTransition> getPreNode(Map<String,List<BPMTransition>> mapLink,BPMTransition targetBPMTransition)
    {
        return mapLink.get(targetBPMTransition.getFrom().getId());
    }

    /**
     *  获取人员的所有部门岗位信息
     * @param currentMember
     * @return
     */
    @Override
    public List<MemberPostVO> getMemberPostList(V3xOrgMember currentMember) throws BusinessException {
        List<MemberPost> postList = new ArrayList<MemberPost>();

        MemberPost mainPost = MemberPost.createMainPost(currentMember);
        postList.add(mainPost);

        List<MemberPost> concurrentPostList = currentMember.getConcurrentPost();
        if(concurrentPostList!= null && !concurrentPostList.isEmpty())
        {
            postList.addAll(concurrentPostList);
        }

        List<MemberPost> secondPostList = currentMember.getSecond_post();
        if(secondPostList!= null && !secondPostList.isEmpty())
        {
            postList.addAll(secondPostList);
        }

        List<MemberPostVO> memberPostVOList = new ArrayList<MemberPostVO>();
        for(MemberPost tempPost : postList)
        {

            Long tempAccountId = tempPost.getOrgAccountId();
            Long tempDeptId = tempPost.getDepId();
            Long tempPostId = tempPost.getPostId();
            V3xOrgAccount account = orgManager.getAccountById(tempAccountId);
            String tempAccountName = "没有设置单位";
            if(account!= null )
            {
                if(Strings.isNotBlank(account.getShortName())){
                    tempAccountName = account.getShortName();
                }else {
                    tempAccountName = account.getName();
                }

            }


            String tempDeptName = "没有设置部门";
            V3xOrgDepartment depart =  orgManager.getDepartmentById(tempDeptId);
            if(depart != null)
            {
                tempDeptName  = depart.getName();
            }

            String tempPostName = "没有设置岗位";
            V3xOrgPost post = orgManager.getPostById(tempPostId);
            if(post!= null)
            {
                tempPostName = post.getName();
            }
            MemberPostVO memberPostVO = new MemberPostVO(tempAccountName,tempDeptName,tempPostName,tempPost);
            memberPostVOList.add(memberPostVO);
        }
        return memberPostVOList;
    }

    /**
     * 获取人员主岗
     * @param currentMember
     * @return
     * @throws BusinessException
     */
    private MemberPostVO getMainMemberPost(V3xOrgMember currentMember){
        try {

            MemberPost mainPost = MemberPost.createMainPost(currentMember);
            Long tempAccountId = mainPost.getOrgAccountId();
            Long tempDeptId = mainPost.getDepId();
            Long tempPostId = mainPost.getPostId();
            V3xOrgAccount account = orgManager.getAccountById(tempAccountId);
            String tempAccountName = "没有设置单位";
            if(account!= null )
            {
                if(Strings.isNotBlank(account.getShortName())){
                    tempAccountName = account.getShortName();
                }else {
                    tempAccountName = account.getName();
                }

            }


            String tempDeptName = "没有设置部门";
            V3xOrgDepartment depart =  orgManager.getDepartmentById(tempDeptId);
            if(depart != null)
            {
                tempDeptName  = depart.getName();
            }

            String tempPostName = "没有设置岗位";
            V3xOrgPost post = orgManager.getPostById(tempPostId);
            if(post!= null)
            {
                tempPostName = post.getName();
            }
            MemberPostVO memberPostVO = new MemberPostVO(tempAccountName,tempDeptName,tempPostName,mainPost);
            return memberPostVO;
        }catch (Exception e){
            return null;
        }
    }

    /**
     * 获取上一节点的落款部门的上级部门是否在当前处理人的部门列表
     * @param departmentId
     * @param memberPostVOList
     * @param allDepartmentMap
     * @return
     */
    public MemberPostVO getParentDepartmentForOpinion(Long departmentId, List<MemberPostVO> memberPostVOList, Map<Long,V3xOrgDepartment> allDepartmentMap)
    {
        if(memberPostVOList == null || memberPostVOList.isEmpty())
        {
            return null;
        }


        V3xOrgDepartment depart = allDepartmentMap.get(departmentId);
        if(depart == null)
        {
            return null;
        }
        //获取上级部门
        return getTartParentDepart(allDepartmentMap,memberPostVOList,depart);


    }

    public MemberPostVO getTartParentDepart(Map<Long,V3xOrgDepartment> allDepartmentMap ,List<MemberPostVO> memberPostVOList,V3xOrgDepartment depart ){

        Long parentId = depart.getSuperior();
        if(parentId != -1)
        {
            V3xOrgDepartment parentDepart = allDepartmentMap.get(parentId);
            if(parentDepart != null)
            {
                for(MemberPostVO memberPostVO:memberPostVOList)
                {
                    if(memberPostVO.getDepId() != null && parentDepart.getId().longValue() == memberPostVO.getDepId())
                    {
                        return memberPostVO;
                    }
                }
                //遍历没有找对目标部门继续递归找
                return getTartParentDepart(allDepartmentMap,memberPostVOList,parentDepart);
            }
            else
            {
                return null;
            }
        }
        else
        {
            return null;
        }


    }

    public Map<Long,V3xOrgDepartment> getALLDepartmentMap(Collection<MemberPostVO> memberPostVOList) {
        Map<Long,V3xOrgDepartment> allDepartMap = new HashMap<Long, V3xOrgDepartment>();
        try {
            for(MemberPostVO memberPostVO : memberPostVOList)
            {
                if(memberPostVO.getDepId() != null)
                {
                    if(allDepartMap.get(memberPostVO.getDepId()) != null)
                    {
                        continue;
                    }

                    V3xOrgDepartment tempDeartment = orgManager.getDepartmentById(memberPostVO.getDepId());

                    if(tempDeartment != null)
                    {
                        getDepartmentDown(tempDeartment,allDepartMap);
                    }
                }

            }
        } catch (BusinessException e) {
            e.printStackTrace();
        }

            return allDepartMap;
    }

    /**
     * 获取部门所有的父节点和子节点
     * @param department
     * @param allDepartMap
     * @return
     */
    public void getDepartmentDown(V3xOrgDepartment department,Map<Long,V3xOrgDepartment> allDepartMap) throws BusinessException {

        allDepartMap.put(department.getId(),department);
        List<V3xOrgDepartment> childsDepartments = orgManager.getChildDepartments(department.getId(), false);
        for(V3xOrgDepartment departmentTemp:childsDepartments)
        {
            allDepartMap.put(departmentTemp.getId(),departmentTemp);
        }

    }

    /**
     * 过滤重合上下级部门节点只保留上级部门
     * @param memberPostVOList
     * @return
     */
    public Collection<MemberPostVO> filterDownDepart(List<MemberPostVO> memberPostVOList)
    {
        List<V3xOrgDepartment> departList = new ArrayList<V3xOrgDepartment>();
        Map<Long,MemberPostVO> mapMemberPost = new HashMap<Long, MemberPostVO>();
        for(MemberPostVO memberPostVO : memberPostVOList)
        {
            if(memberPostVO.getDepId() == null)
            {
                continue;
            }

            try {
                V3xOrgDepartment departMent = orgManager.getDepartmentById(memberPostVO.getDepId());
                if(departMent!= null)
                {
                    departList.add(departMent);
                    mapMemberPost.put(memberPostVO.getDepId(),memberPostVO);
                }
            } catch (BusinessException e) {
                e.printStackTrace();
            }
        }


        for(V3xOrgDepartment department : departList)
        {
            boolean flag = false;
            String path = department.getPath();
            for(V3xOrgDepartment departmentCompare : departList)
            {
                if(departmentCompare.getId() != department.getId().longValue()  && path.contains(departmentCompare.getPath()))
                {
                    //表示path是子节点不需要查找
                    flag = true;
                    break;
                }

            }
            if(flag)
            {
                mapMemberPost.remove(department.getId());
            }
        }
        return mapMemberPost.values();
    }

    /**
     *  评论时间倒序
     */
    public void sortCommentListDesc(List<Comment> commentList){
        Collections.sort(commentList,new Comparator<Comment>() {
            @Override
            public int compare(Comment before, Comment after) {


                if(after.getCreateDate() == null && before.getCreateDate()  == null)
                {
                    return  0;
                }

                if(after.getCreateDate() == null && before.getCreateDate() != null)
                {
                    return  -1;
                }

                if(after.getCreateDate() != null && before.getCreateDate() == null)
                {
                    return  1;
                }

                long beforeCreate = before.getCreateDate().getTime();
                long afterCreate = after.getCreateDate().getTime();

                if(afterCreate> beforeCreate)
                {
                    return 1;
                }
                if (afterCreate < beforeCreate)
                {
                    return -1;
                }

                return 0;
            }
        });
    }
}
